<?xml version="1.0" encoding="iso-8859-1"?>
<!--
	Page for display of partition and file system information


  -->


<!DOCTYPE html [
  <!ENTITY nbsp "<xsl:text disable-output-escaping='yes'>&amp;nbsp;</xsl:text>">
  <!ENTITY ampersand "<xsl:text disable-output-escaping='yes'>&amp;</xsl:text>">
]>


<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <!-- [ ! ] --><xsl:include href="common.xsl"/>
  <!-- <xsl:include href="../vmware/common/operation-btn.xsl"/> -->

  <xsl:template match="/viewPart">
    <HTML>
      <HEAD>
	<TITLE>VMware ESX Server Configuration -- Partition/VMFS Editor</TITLE>
      </HEAD>

      <BODY bgcolor="#FFFFFF" link="#000000" vlink="#000000" topmargin= "6" leftmargin="6" marginwidth="6" marginheight="6">

	<xsl:if test="sys">

	  <H1>VMware ESX Server Configuration</H1>
	  <H2>Partition/VMFS Editor</H2>


	  <!--
	    |  Display generic system information
	    -->

	  <xsl:if test="sys/systemInfo">
	    <P>
	    System information: <xsl:value-of select="sys/systemInfo"/>.
	    </P>
	  </xsl:if>


	  <!--
	    |  Check if the VM kernel is running
	    -->

	  <xsl:if test="sys/VMKernel">
	    <P>
	    VMKernel running: <xsl:value-of select="sys/VMKernel"/>.
	    </P>
	  </xsl:if>

	  <xsl:if test="sys/kernelWarnings">
	    <H2>VM kernel check warnings</H2>
	    <UL>
	      <xsl:for-each select="sys/kernelWarnings/*[@msgType=warning]">
		<LI><xsl:value-of select="."/></LI>
	      </xsl:for-each>
	    </UL>
	    <P>
		The VMkernel must be loaded in order to perform this step.
		Please load the VMkernel before doing this step.  You may also
		have to load in a SCSI VMkernel module in order to edit any
		partitions.  You may skip this step if you do not wish to use
		the VMFS file system.
	    </P>
	  </xsl:if>


	  <!--
	    |  Check if all the VM kernel modules are loaded
	    -->

	  <xsl:if test="sys/moduleStatus">
	    <P>
	    Module status: <xsl:value-of select="sys/moduleStatus"/>.
	    </P>
	  </xsl:if>

	  <xsl:if test="sys/moduleWarnings">
	    <H2>Module check warnings</H2>
	    <UL>
	      <xsl:for-each select="sys/moduleWarnings/*[@msgType=warning]">
		<LI><xsl:value-of select="."/></LI>
	      </xsl:for-each>
	    </UL>
	    <P>
		The VMkernel modules need to be loaded before proceeding.
	    </P>
	  </xsl:if>


	  <!--
	    |  Check for the existence of a dump partition
	    -->

	  <xsl:if test="sys/dumpPartitions">
	    <P>
	    <xsl:variable name="numPartitions" select="sys/dumpPartitions"/>
	    <xsl:choose>
	      <xsl:when test='$numPartitions=1'>
		<xsl:value-of select="$numPartitions"/>
		dump partition found.
	      </xsl:when>
	      <xsl:otherwise>
		<xsl:value-of select="$numPartitions"/>
		dump partitions found.
	      </xsl:otherwise>
	      </xsl:choose>
	    </P>
	  </xsl:if>

	  <xsl:if test="sys/diskWarnings">
	    <H2>Disk check warnings</H2>
	    <UL>
	      <xsl:for-each select="sys/diskWarnings/*[@msgType=warning]">
		<LI><xsl:value-of select="."/></LI>
	      </xsl:for-each>
	    </UL>
	    <P>
		VmkDump partition does not exist.  It is recommended that you
		create a 50 megabyte partition that will be used to contain core
		dump and log files.  You can create such a partition by using
		Fdisk.  If you have a disk without partitions, you can use the
		'Create New Partition' button to have one created for you
		automatically.
	    </P>
	  </xsl:if>


	  <!--
	    |  Display a partition table for each disk, if there are SCSI
	    |  disks found.
	    -->

	  <xsl:if test="sys/SCSIDisks/count">
	    <H2>Editing Partitions and Creating VMFS File Systems</H2>

	    <P>
		Here you can setup your SCSI disks to use the VMFS file
		system that allows you to have non-persistent and redoable
		disks.  To setup an VMFS file system, you can either convert
		an existing partition or create new a new partition for VMFS
		on this page.
	    </P>
	    <P>
		If you have any partitions defined, you can convert them to
		VMFS file systems.  To make a VMFS file system, click on
		"Make VMFS" button next to each partition.  If you have any
		extended partitions, they will not be displayed.
	    </P>
	    <P>
		You may also create and delete partitions on your SCSI disk
		from this page by clicking on the appropriate button below
		each disk listing.
	    </P>
	    <P>
		The following disks were detected on your system:
	    </P>

	    <xsl:for-each select="sys/*[@diskName]">
	      <UL>
		<LI>  <xsl:value-of select="@diskName"/></LI>
	      </UL>
	    </xsl:for-each>

	    <xsl:for-each select="sys/*[@diskName]">
	      <H3>Disk <xsl:value-of select="@diskName"/></H3>
	      <xsl:if test="partitions/count">
		<TABLE border="0" cellspacing="4">
		<TR>
		  <TD>&nbsp;&nbsp;&nbsp;</TD>
		  <TD>
		    <TABLE border="1" cellpadding="2">
		      <TR align="left">
			<TD><B>Partition</B></TD>
			<TD><B>Type</B></TD>
			<TD><B>Start Cylinder</B></TD>
			<TD><B>End Cylinder</B></TD>
			<TD><B>Size (MB)</B></TD>
			<TD><B>System</B></TD>
			<TD>&nbsp;</TD>
		      </TR>

		      <xsl:for-each select="*[@number]">
			<TR>
			  <TD><xsl:value-of select="@number"/></TD>
			  <TD align="center"><xsl:value-of select="type"/></TD>
			  <TD align="center"><xsl:value-of select="startCyl"/></TD>
			  <TD align="center"><xsl:value-of select="endCyl"/></TD>
			  <TD align="center"><xsl:value-of select="MB"/> MB</TD>
			  <TD><xsl:value-of select="typeName"/></TD>
			  <TD>&nbsp;</TD>
			</TR>
		      </xsl:for-each>

		    </TABLE>
		  </TD>
		</TR>
		</TABLE>
	      </xsl:if>
	    </xsl:for-each>
	  </xsl:if>

	</xsl:if>

      </BODY>

    </HTML>
  </xsl:template>



  <xsl:template match="/Error">
	<H1>VMware ESX Server Error</H1>
	<P><xsl:value-of select="Message"/></P>
  </xsl:template>


</xsl:stylesheet>

