#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# RunlevelControl.pm
# 
# Helper functions that start and stop services that are controlled 
# via the init scripts (/etc/rc.d).


package VMware::HConfig::RunlevelControl;
use strict;
use VMware::ExtHelpers qw(&Warning &shell_string &System &mktmpfile &safe_move);
use VMware::Tokenizer;
use VMware::HConfig::Constant qw(c_getFile c_getDir c_getHelper);
				 


#####################################################################
# Public Functions                                                  #
#####################################################################


sub runLevelStart($) {
  my ($service) = @_;

  if (!setDefaultState($service, "on")) {
     Warning("failed to set default service state");
     return(0);
  }

  if (System(c_getDir('init_scripts') . '/'. $service . ' restart')) {
     Warning("Was not able to start $service");
     return(0);
  }
  return(1);
}

sub runLevelStop($) {
  my ($service) = @_;
  if (!setDefaultState($service, "off")) {
     Warning("failed to set default service state");
     return(0);
  }

  if (System(c_getDir('init_scripts') . '/' . $service . ' stop')) {
     Warning("Was not able to stop $service");
     return(0);
  }
  return(1);
}


sub runLevelQuery($) {
  my ($service) = @_;
  local *FD;
  my $listCmd;

  $listCmd = c_getHelper('host_service_list') . " " . $service . "|";

  if (!open(FD,$listCmd)) {
     Warning("Could not execute $listCmd.");
     return(0);
  }

  #XXX should query current run level -- not assume that we are
  #in run level 3
  while (<FD>) {
     if (/3:(on|off)/) {
        close(FD);
        return $1;
     }
  }
  close(FD);
  Warning("Improper reply from " . c_getHelper('host_service_list'));
  return(0);
}

#####################################################################
# Private Functions                                                 #
#####################################################################


# Use chkconfig to set the default state of a service
sub setDefaultState($$) {
  my ($service,$state) = @_;

  if (System(c_getHelper('host_service_set') .  " " .
             $service . " $state") ) {

     Warning("Execution of " . c_getHelper('host_service_set') . " failed. Reason: $!");
     return(0);
  }
  return(1);
}


1;
