#############################################################
# Copyright 2003 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################

# File that defines drivers that have overlapping PCI IDs.  The data
# structure below defines a map from a display string used by the MUI
# to a list of device drivers and a name.  Every driver in a list will
# have at least 1 PCI ID in common with that of at least 1 other driver.
# The name field refers to the type of device once a driver is loaded
# into the vmkernel. It is not used in the MUI display, but rather to 
# determine "device equivalence".  Two devices with the same NAME field
# below, if physically interchanged, will be assigned the same device
# name by the vmkernel.
#
# These driver sets are defined statically for the MUI which will
# present devices depending on the drivers selected by the user.
#
# Note: if you get an error after making changes to the device map file,
# the data structure below may need to be modified.  The table below
# is validated against the data in the vmware-devices.map file.

%VMK_MODULE_SET = (
   "3Com 90x Network Adapters" => { NAME => "nic",
                                    DRIVERS => [ "3c90x.o" ] },
   "3Com 99x Network Adapters" => { NAME => "nic",
                                    DRIVERS => [ "3c990.o" ] },
   "EtherExpress 100 MB" =>       { NAME => "nic",
                                    DRIVERS => [ "e100.o" ] },
   "EtherExpress 1 GB" =>         { NAME => "nic",
                                    DRIVERS => [ "e1000.o" ] },
   "Acenic" =>                    { NAME => "nic",
                                    DRIVERS => [ "acenic.o" ] },
   "Broadcom" =>                  { NAME => "nic",
                                    DRIVERS => [ "bcm5700.o" ] },
   "Adaptec AIC 7xxx" =>          { NAME => "aic7xxx",
                                    DRIVERS => [ "aic7xxx.o" ] },
   "Adaptec AIC 79xx" =>          { NAME => "aic79xx",
                                    DRIVERS => [ "aic79xx.o" ] },
   "Symbios 53c8xx" =>            { NAME => "sym53c8xx",
                                    DRIVERS => [ "sym53c8xx.o", "ncr53c8xx.o" ] },
   "BusLogic" =>                  { NAME => "BusLogic",
                                    DRIVERS => [ "BusLogic.o" ] },
   "DAC 960 RAID" =>              { NAME => "DAC960",
                                    DRIVERS => [ "DAC960.o" ] },
   "Adaptec RAID" =>              { NAME => "aacraid",
                                    DRIVERS => [ "aacraid.o" ] },
   "IBM Megaraid" =>              { NAME => "megaraid",
                                    DRIVERS => [ "megaraid.o" ] },
   "Compaq Array" =>              { NAME => "cpqarray",
                                    DRIVERS => [ "cpqarray.o" ] },
   "CCISS" =>                     { NAME => "cciss",
                                    DRIVERS => [ "cciss.o" ] },
   "QLogic 2200" =>               { NAME => "fc",
                                    DRIVERS => [ "qla2200_604.o", "qla2200_603.o" ] },
   "QLogic 2300" =>               { NAME => "fc",
                                    DRIVERS => [ "qla2300_604.o", "qla2300_603.o" ] },
   "MPTSCSI" =>                   { NAME => "mptscsi",
                                    DRIVERS => [ "mptscsi.o" ] },
   "IPS" =>                       { NAME => "ips",
                                    DRIVERS => [ "ips.o" ] },
   "Emulex LPxxxx" =>             { NAME => "fc",
                                    DRIVERS => [ "lpfcdd.o", "lpfcdd_122.o" ] },
   "DPT_I2O" =>                   { NAME => "dpt_i2o",
                                    DRIVERS => [ "dpt_i2o.o" ] },

);
