# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::Answer;
# File: VMware/Management/Answer.pm

use strict;

use VMware::Control '1.00';
use VMware::Control::Server;
use VMware::Control::VM;
use VMware::Control::Profiler;
use VMware::Management::Util;
use VMware::Management::Dialog ();
use VMware::Management::MultipartCache;

use CGI ();
use XML::Sablotron ();

my $q;
my $p = VMware::Control::Profiler::getInstance("mui");

sub handler {
    my $stub = $p->profile();
  $q = new CGI();
  trace( "ANSWER: ".$q->param( "part" )." ".$q->param( "status" ), "debug");
  if ( $q->param( "multipart-key" ) ) {
    return emit_xml( );
  }
  my %credentials = hash_credentials( );
  my ( $server, $server_ok, $vm, $vm_ok, $err, $errstr );

  $server = &VMware::Control::Server::new( undef, 0, $credentials{ "l" }, $credentials{ "m" } );
  ( $server_ok, $err, $errstr ) = connect_ctrl_obj( $server );
  unless( $server_ok ) {
    # $server is not connected, so why call cleanup( )?
    return emit_xml( $vm, $err, $errstr );
  }

  $vm = VMware::Control::VM::new( $server, $q->param( 'vm' ) );
  ( $vm_ok, $err, $errstr ) = connect_ctrl_obj( $vm );
  unless( $vm_ok ) {
    cleanup( undef, $server ); # $server is ok if we get here.
    return emit_xml( undef, $err, $errstr );
  }
  
  if ( $q->param( "status" ) eq "confirmed" ) {
    my $e = $vm->answer_question( $q->param( "question-id" ), $q->param( "answer-id" ) );
    trace( "vm->answer_question( ) called.", "notice" );
    unless ( $e ) {
      my ( $err, $errstr ) = $vm->get_last_error( );
      trace( $err, "error" );
      trace( $errstr, "error" );
      cleanup( undef, $server ); # $vm and $server are ok if we get here.
      return emit_xml( $vm, $err, $errstr );
    }
  }

  cleanup( undef, $server ); # $vm and $server are ok if we get here.
  return emit_xml( $vm );
}

sub cleanup {
    my $stub = $p->profile();
  my ( $vm, $server ) = @_;
  if ( $vm ) {
    $vm->disconnect( );
    trace( "vm->disconnect( ) called.", "notice" );
  }

  if ( $server ) {
    $server->disconnect( );
    trace( "server->disconnect( ) called.", "notice" );
  }
}

sub emit_xml {
    my $stub = $p->profile();
  my ( $vm, $e, $s ) = @_;
  my $processor = new XML::Sablotron();
  my $xsl = $ENV{ "style_DEFAULT" };
  my $data;
  my $dlg;

  if ( $q->param( "multipart-key" ) ) {
    $data = retrieve_part( $q->param( "multipart-key" ) );
  } else {
    $dlg = new VMware::Management::Dialog( );
    my $qid = $q->param( "question-id" );

    # [ ! ] Question queing is busted. Check if there is really a question to
    # be answered.
    unless ( $vm && defined( $vm->get( "Status.question.current" ) ) ) {
      unless ( $e != -16 || !$e ) {
        trace( "Status.question.current is undefined - ignoring waiting for input error.", "debug" );
        $e = undef;
        $qid = -1;
      }
    }

    my $cfg = $vm->get( "Config.filename" ) unless !$vm;
    $cfg ||= $q->param( "vm" );
    my $qtxt = $vm->get( "Status.question.$qid.text" ) unless ( !$vm || $qid < 0 );
    my $display_name = $vm->get( "Config.displayName" ) unless !$vm;

    if ( $q->param( "status" ) eq "pending" && !$e && $qid > -1 ) {
      $dlg->set_type( "prompt" );
      $dlg->set_title( "Waiting for Input:" );
      my $node = $dlg->set_vm( );
      $dlg->set_vm_cfg( $node, $cfg, $display_name );
      $node = $dlg->set_operation( "answer" );
      $dlg->set_operation_status( $node, "pending" );
      $dlg->set_description( $qtxt  );  

      # Add question form data to the output.
      $dlg->append_action( "question",
                           "",
                           "question",
                           "QUESTION" );

      $node = $dlg->append_args( "QUESTION" );
      $dlg->append_input( $node, "status", $q->param( "status" ), "true" );
      $dlg->append_input( $node, "question-id", $q->param( "question-id" ), "true" );
      $dlg->append_input( $node, "vm", $q->param( "vm" ), "true" );

      # Add answer form data to the output.
      $dlg->append_action( "answer",
                           "",
                           "answer",
                           "ANSWER" );

      $node = $dlg->append_args( "ANSWER" );
      $node = $dlg->append_select( $node, "answers", "exclusive" );

      for ( my $i = 0; $i < $vm->get( "Status.question.$qid.choices.number" ); $i++ ) {
        $dlg->append_option( $node,
        $vm->get( "Status.question.$qid.choices.$i" ),
        $i );
      }
    } else {
      if ( !$e ) {
        $dlg->set_type( "notice" );
        $dlg->set_title( "Waiting for Input: Success" );
        my $node = $dlg->set_vm( );
        $dlg->set_vm_cfg( $node, $cfg, $display_name );
        $node = $dlg->set_operation( "answer" );
        $dlg->set_operation_status( $node, "confirmed" );
        # [ ! ] It would be good to say how the VM is continuing. That is,
        # echo back to the user his or her answer.
        $dlg->set_description( "Continuing as requested." );
      } else {
        $dlg->set_type( "alert" );
        my $node = $dlg->set_vm( );
        $dlg->set_vm_cfg( $node, $cfg, $display_name );
        if ( $e == -16 && $vm ) {
          $dlg->append_input( $node, "error-id", $e, "true" );
          $dlg->append_input( $node, "question-id", $vm->get( "Status.question.current" ), "true" );
        }
        $node = $dlg->set_operation( "answer" );
        $dlg->set_operation_status( $node, "error" );
        $dlg->set_title( "Error" );
        $dlg->set_description( $s );
      }
    }

    if ( $ENV{ 'style_DEFAULT_PARTS' } && $e != -16 ) {
      my @parts = split( /\|/, $ENV{ 'style_DEFAULT_PARTS' } );
      
      foreach my $part ( @parts ) {
        my $key = generate_part( $dlg->to_string( ) );
        $dlg->append_multipartKey( $part, $key );
      }
    }
    $data = $dlg->to_string( );
    $dlg->dispose( );
  }

  my $xml = "/tmp/answer.xml";
  if ( $ENV{ "vmware_DEBUG_LEVEL" } == 3 ) {
    string_to_file( $data, $xml, 1 );
  }

  cleanup( $vm );

  $processor->SetBase( "file://" . $ENV{ "vmware_BASEHREF" } );
    $p->start("XML::Sablotron::RunProcessor");
  $processor->RunProcessor( "arg:/template", "arg:/data", "arg:/result",
                            undef,
                            [ "template", file_to_string( $xsl ), "data", $data ] );
    $p->end();
  return ( $processor->GetResultArg("result"),
           "Content-Type"  => "text/html; charset=iso-8859-1",
           "Pragma"        => "no-cache",
           "Cache-Control" => "no-cache" );
}

1;
