#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# ConfigHTML.pm
# 
# A collection of common routines for generating config pages. 

package VMware::Management::ConfigHTML;

use VMware::Control::Server;
use VMware::Management::Util qw( hash_credentials );
use VMware::DOMAccess;

my $themedir = "/vmware/themes/default";
my $spacer = "/vmware/imx/spacer.gif";
my $imxdir = "$themedir/wgs/imx";
my $URL_INDEX = "/config/";
my $URL_LOGOUT = "/logout";
my $URL_OVERVIEW = "/overview";

sub PanicCallbackHTML(@) {
    my (@error) = @_;
    my $message = "";
    if ( $error[0] ) {
	$message = $error[0];
    }
    print toErrorHTML("VMware Config Internal Failure",
		      $message);
}

sub toErrorHTML($$$) {
    my ($eSummary, $eDesc, $errorTitle) = @_;
	my $pathbar = toHTMLPath((), "Error");
    my $s =
	"<html>\n" .
        "<head>\n" .
        "<title> $errorTitle -- Error $eSummary</title>\n" .
        "</head>\n" .
        "\n" .
        "<body bgcolor=\"#ffffff\"><hr>\n" .

        "<h3>Error: $eSummary</h3>\n" . 
        "<p>\n" . 
	$eDesc . "\n" .
        "</p>\n" . 

        "<hr></body>\n" . 
        "</html>\n";

    return($pathbar . $s);
}

sub toHTMLHeader($;$$) {
   my ($title, $username, $meta) = @_;
   if (!defined($meta)) {
      $meta = "";
   }

   if (defined($username)) {
      $username = "Management Interface for $username";
   } else {
      $username = "Management Interface";
   }

   my $build = 'Build: 44356';

  # my $s = "header";
   my $s = <<END_HTML;
<html>
<head>
  <title>$title</title>
  <link rel="stylesheet" 
        href="$themedir/style.css" 
        type="text/css">
  $meta

  <script type="text/javascript">
    <!--
      function aggressiveUpdate() {
        window.setTimeout('window.location.reload( )', 1000 * 60);
      }
      function openHelp(ref,w,h) {
         window.open(ref,
                     "vmware_mui_help",
                     "location,menubar,resizable,scrollbars,status,toolbar," +
                     "width=" + w + ",height=" + h);
         aggressiveUpdate();
      }
    -->
  </script>
</head>
<body bgcolor="#ffffff" topmargin="6" leftmargin="6" marginwidth="6" marginheight="6"
      link="#3366cc" vlink="#663300">
  <table width="100%" border="0" cellpadding="0" cellspacing="0">
    <tr>
      <td width="1%" nowrap="true"><img src="$imxdir/header.gif" width="451" height="62"></td>
      <td align="right" valign="top" nowrap="true" background="$imxdir/header-bg.gif">
        <font color="#ffffff"><small><small>
          <br><nobr>$username&nbsp;</nobr>
          <br><nobr>$build&nbsp;</nobr>
        </small></small></font>
      </td>
    </tr>
  </table>
END_HTML
   return $s;
}


sub toHTMLFooter() {
   my $s = <<END_HTML;
  <table width="100%" border="0" cellpadding="0" cellspacing="0">
    <tr>
      <td align="right" nowrap="true" background="$imxdir/footer-bg.gif">&nbsp;</td>
      <td width="1%" nowrap="true"><img src="$imxdir/footer.gif" width="451" height="49"></td>
    </tr>
  </table>

  <p align="center"><small><small>&copy; 1998-2006 VMware, Inc. All rights reserved. U.S.<br />
  Protected by one or more of U.S. Patent Nos. 6,397,242, 6,496,847, 6,704,925, 6,711,672, 6,725,289, 6,735,601, 6,785,886, 6,789,156, 6,795,966, 6,880,022 6,961,941, 6,961,806 and 6,944,699; patents pending.
  </small></small></p>
</body>

</html>
END_HTML

   return $s;
}

sub toHTMLPath(@$) {
   my ($locations, $title) = (@_);

   my $s = <<END_HTML;
  <table width="100%" border="0" cellpadding="1" cellspacing="0">
    <tr>
      <td><img src=$spacer width="1" height="4"></td>
    </tr>
    <tr>
      <td bgcolor="#999980">
        <table width="100%" border="0" cellpadding="3" cellspacing="0">
          <tr bgcolor="#CCCCB3">
            <td>
              <table border="0" cellpadding="3" cellspacing="0">
                <tr>
END_HTML

   foreach my $location (@$locations) {
      my ($title, $url) = @$location;
      my $entry;
      if (defined($url)) {
         $entry = qq(<a href = "$url">$title</a>);
      } else {
         $entry = $title;
      }

      $s .= <<END_HTML;
                  <td><small><small><nobr>$entry</nobr></small></small></td>
                  <td><small><small><nobr>|</nobr></small></small></td>
END_HTML
   }

   $s .= <<END_HTML;
                  <td><nobr><small><small><b>$title</b></small></small></nobr></td>
                </tr>
              </table>
            </td>
END_HTML

   # XXX rbosch: Should only show overview button if VMkernel is loaded
   $s .= qq(<td width="1%" align="right">\n) .
           toHTMLButton("xxs", "VM Overview", $URL_OVERVIEW) .
           qq(</td>\n);

   my $url_help = "http://www.vmware.com/support/gsx2/doc/manage_gsx.html";
   $url_help = qq(javascript:openHelp('$url_help', 700, 500););
   $s .= qq(<td width="1%" align="right">\n) .
           toHTMLButton("xxs", "Help", $url_help) .
           qq(</td>\n);

   $s .= qq(<td width="1%" align="right">\n) .
           toHTMLButton("xxs", "Logout", $URL_LOGOUT) .
           qq(</td>\n);

   $s .= <<END_HTML;
          </tr>
        </table>
      </td>
    </tr>
  </table>
END_HTML

   return $s;
}


sub toHTMLButton {
   my ($class, $text, $action, $onClick) = @_;

   if (defined($action) && $action ne "") {
      $action = qq(href="$action");
   } else {
      $action = qq(name="$text");
   }

   if (defined($onClick) && $onClick ne "") {
      $action .= qq( onClick="$onClick");
   }

   my $textColor = "#000000";
   my ($baseColor, $gutterShadow, $gutterHilite, $edgeShadow, $edgeHilite) =
       ("#ccccb3", "#999980", "#ffffe6", "#999980", "#ffffe6");
       #("#e6e6cc", "#ccccb3", "#ffffe6", "#ccccb3", "#ffffe6");

   return <<END_HTML;
<table border="0" cellpadding="0" cellspacing="0">
  <tr><!-- 9 : 7 -->
    <td colspan=7 bgcolor=$gutterShadow><img src="$spacer" width=1 height=1></td>
  </tr>
  <tr><!-- 9 : 6 --><!-- 0 : 5 --><!-- F : 5 -->
    <td rowspan=6 bgcolor=$gutterShadow><img src="$spacer" width=1 height=1></td>
    <td colspan=5 bgcolor="#000000"><img src="$spacer" width=1 height=1></td>
    <td rowspan=5 bgcolor=$gutterHilite><img src="$spacer" width=1 height=1></td>
  </tr>
  <tr><!-- 0 : 4 --><!-- F : 3 --><!-- 0 : 3 -->
    <td rowspan=4 bgcolor="#000000"><img src="$spacer" width=1 height=1></td>
    <td colspan=3 bgcolor=$edgeHilite><img src="$spacer" width=1 height=1></td>
    <td rowspan=3 bgcolor="#000000"><img src="$spacer" width=1 height=1></td>
  </tr>
  <tr><!-- F : 2 --><!-- C : 1 --><!-- 9 : 1 -->
    <td rowspan=2 bgcolor=$edgeHilite><img src=$spacer width=1 height=1></td>
    <td bgcolor=$baseColor>
      <table border=0 cellpadding=1 cellspacing=0>
        <tr>
          <td align="center" nowrap="true" class=$class><nobr>
            <a style="text-decoration:none;" $action>
              <font color=$textColor>&nbsp;&nbsp;$text&nbsp;&nbsp;</font>
            </a>
          </nobr></td>
        </tr>
      </table>
    </td>
    <td bgcolor=$edgeShadow><img src=$spacer width=1 height=1></td>
  </tr>
  <tr><!-- 9 : 2 -->
    <td colspan=2 bgcolor=$edgeShadow><img src=$spacer width=1 height=1></td>
  </tr>
  <tr><!-- 0 : 4 -->
    <td colspan=4 bgcolor="#000000"><img src=$spacer width=1 height=1></td>
  </tr>
  <tr><!-- F : 6 -->
    <td colspan=6 bgcolor=$gutterHilite><img src=$spacer width=1 height=1></td>
  </tr>
</table>

END_HTML


}

my $server = undef;
sub request($) {
    my $doc = shift;
    my $err;
    my %credentials = hash_credentials( );

    my $xml = $doc->getValue("");

    if(!defined($server)) {
       $server = &VMware::Control::Server::new(undef, 0, $credentials{ "l" }, $credentials{ "m" } );

	   if (!$server || !$server->connect()) {
	      Warning("Cannot connect", "debug");
	   }
    }

    my $ret = $server->exec($xml);
    if (!$ret) {
       my ($err, $errstr) = $server->get_last_error();
	   Warning("VMControl error $err: $errstr\n", "error");
    }
	    
    my $resp = VMware::DOMAccess->newXML($ret);

    return $resp;
}

sub isAdminUser() {
   my $in = VMware::DOMAccess->new("exec");

   $in->setValue("exec.op", "VMServerd_IsAdminUser");

   my $out = &VMware::Management::ConfigHTML::request($in);
   my $retval = $out->getValue(".out.adminuser");

   $in->dispose();
   $out->dispose();

   return $retval;
}


1;
