# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::Console;
# File: VMware/Management/Console.pm

use strict;
use CGI;
use VMware::Control::Profiler;
use VMware::Management::Util qw( trace );

my $p = VMware::Control::Profiler::getInstance("mui");

sub handler {
  my $stub = $p->profile();

  my $q = new CGI(shift());

  my $xvm = $q->param( "vm" );
  trace( "xvm '$xvm', param(c) '". $q->param("c"). "'", "debug" );
  $xvm ||= $q->param( "c" );
  my $cfg = $xvm;
  trace( "cfg '$cfg'", "debug" );
  
  # There doesn't seem to be any good way to convince IE4 that the filename is
  # *.xvm short of modifying the URL so server.js appends a .xvm to the end of
  # the config file argument. Make sure to splice .vmx/.cfg extensions out of
  # the .xvm file name (for use in the Content-Type header) and to strip the
  # .xvm extension off the end of the config file (for use in the remote
  # console options file).
  if ($^O eq "MSWin32") {
    $xvm =~ s/^.*\\(.*?)(\.cfg|\.vmx)(\.xvm)$/$1$3/;
  } else {
    $xvm =~ s/^.*\/(.*?)(\.cfg|\.vmx)(\.xvm)$/$1$3/;
  }
  $cfg =~ s/\.xvm$//;
  trace( "cfg '$cfg'", "debug" );

  my $self_url = VMware::Management::Util::get_cgi_hostname($q);
  if (!$self_url) {
      trace("Console.pm: Failed to get MUI hostname.\n", "error");
      $self_url = "localhost";
  }
  my $port = VMware::Control::get_default_tcp_port();

  trace( "xvm '$xvm'", "debug" );
  my %credentials = VMware::Management::Util::hash_credentials( );

  return ( "-h $self_url -P $port -u $credentials{l} -- \"$cfg\"",
           "Content-Type"  => "application/x-vmware-console; name=\"$xvm\"",
           "Cache-Control" => "must-revalidate" );
}

1;
