#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# GSXSecurityConfig.pm
# 
# A script that presents the setup options available.
package VMware::Management::GSXSecurityConfig;

use strict;
use VMware::ExtHelpers qw(&Warning 
                          $gPanicCallback 
			  &clearWarnings);
use Carp qw(confess);
$SIG{__DIE__} = \&confess;
use CGI;
use VMware::Control '1.00';
use VMware::Control::Server;
use VMware::Control::VM;
use VMware::Management::ConfigHTML;
use VMware::DOMAccess;

my $RCom = "request";
my $RCom_Save = "savesecurity";
my $encryptRUI =  "ruissl";
my $encryptMUI =  "muissl";
my $htmlTitle = "Security Configuration";
my $user;
    
my $URL_INDEX = "/config/index.pl";
my $URL_SELF = "/securityconfig";
my @htmlPath = ( [ "VMware GSX Server Configuration" , $URL_INDEX ] );

#####################################################################
# HTML producing functions                                          #
#####################################################################


# This function displays a page to make the user wait until
# the httpd server has restarted.
sub toWaitingForServerHTML($$$$$) {
   my ($cgi, $nextPage, $title, $msg, $timeOut) = @_;
   my $s = "";
   
   $s .= 
   ("<HTML><HEAD><TITLE>Waiting for operation to complete</TITLE>\n" .
    "<META http-equiv=\"Refresh\" content=\"$timeOut;url=$nextPage\"</HEAD>\n" .
    "<body bgcolor=\"#ffffff\">\n" .

    "<hr>\n" .
    "<h3>$title</h3>\n" . 
    "<p class=\"txt\">Please wait for the operation to complete.  $msg, click <a href=\"$nextPage\">here</a> to continue.\n" .
    "</p>\n" .
    "<hr>\n" .

    "</BODY></HEAD>\n");
}

sub updateConfigVar($$$) {
   my ($cgi, $key, $options) = @_;

   $options->{$key} = ($cgi->param($key) eq "on") ?  "yes" : "no";
}


sub usingNTHost() {
   my $in = VMware::DOMAccess->new("exec");
   $in->setValue("exec.op", "GSXHostInfo_Query");
   my $out = &VMware::Management::ConfigHTML::request($in);

   my $hostOSval = $out->getValue(".out.hostOS");
   my $retval = $out->getValue(".out.hostOSVersion");

   $in->dispose();
   $out->dispose();

   if ($hostOSval eq "MSWin32") {
      return (($retval lt 5) ? 1 : 0);
   } else {
      return 0;
   }
}

sub getResultsString($$) {
   my ($name, $value) = @_;
   my $s = "";

   if ($value eq "yes") {
      $s .= "<li>Enabling ";
   } else {
      $s .= "<li>Disabling ";
   }
   if ($name eq $encryptMUI) {
      $s .= "SSL for the Management Interface.  ";
   } else { 
      #$name eq $encryptRUI
      $s .= "SSL for the Remote Console.  ";
   } 
}

#
# saveConfig()
#
# Make a serverd request to save the security settings,
# and activate the changes.
sub saveConfig($) {
    my ($cgi) = @_;
    my %options;
    my %results;
    my $timeOut;
    my $s = "";
    my $errorList = "";
    my $successList = "";
    my $key;

    if (!usingNTHost()) {
       updateConfigVar($cgi, $encryptMUI, \%options);
    }
    updateConfigVar($cgi, $encryptRUI, \%options);
    if (!activateChanges(\%options, \%results)) {
       $s .= &VMware::Management::ConfigHTML::toErrorHTML("SSL Internal Error ",
             "<p class=\"txt\">Please file an incident report with VMware support.</p>\n", 
             $htmlTitle);
       return $s;
    } 

    # Go through the results of activateChanges and check that all calls to 
    # enable/disable SSL were successful.  If a test fails, add it to a list 
    # of calls which get printed out to an error page. 
    foreach $key (keys %results) {
       if (!$results{$key}) {
	      # Failed to enable SSL for this key.
          $errorList .= getResultsString($key, $options{$key});
       } else {
          # The call was successful.
          $successList .= getResultsString($key, $options{$key});
       }
    }  

    if ($errorList ne "") {
       # An error occurred. Go to error page. 
       my $errorMsg = "<p class=\"txt\"> The following operations failed: " .
	                                       "<ul class=\"txt\">$errorList</ul> </p>";
       if ($successList ne "") {
          # Also list the operations which succeeded.
          $errorMsg .= "<p class=\"txt\"> The following operations succeeded: " .
		                                   "<ul class=\"txt\">$successList</ul> </p>";
       } 
       $s .= &VMware::Management::ConfigHTML::toErrorHTML("SSL Changes Failed to Commit ",
                                                          $errorMsg, $htmlTitle);
       return $s;
    }

    my $nextUrl = $cgi->url();
    if ($^O eq "MSWin32") {
       if ($results{$encryptMUI}) {
          #kind of a hack, but I don't know how to do the neat server side
          #url rewriting that apache does.
          if ($options{$encryptMUI} eq "yes") {
             if ($nextUrl =~ s/8222/8333/) {
                $nextUrl =~ s/^http/https/;
             }
          } else {
             $nextUrl =~ s/8333/8222/;
             $nextUrl =~ s/^https/http/;
          }
       }
       $timeOut = 1; 
    } else {
       $timeOut = 12;  #must be long enough for apache to restart
    }
    $nextUrl =~ s/$URL_SELF.*$/$URL_INDEX/;  
    $s .= &VMware::Management::ConfigHTML::toHTMLPath(\@htmlPath, $htmlTitle);
    $s .= toWaitingForServerHTML($cgi, $nextUrl,
             "Applying security changes ...",
             "If this page doesn't automatically reload within 15 seconds", $timeOut);
    return $s;
}


sub remoteUISecurity($){
   my ($checked) = @_;
   my $s = "";
   $s .= 
        ("<h3>Remote Console</h3>\n"  .
        "<table border=0>\n" .
        "<tr>\n" .
        "   <td><input type=\"checkbox\" name=\"$encryptRUI\" $checked->{$encryptRUI}></td>\n" .
        "   <td class='txt'>Use Secure Sockets Layer (SSL) to encrypt remote console connections</td>\n" .
        "</tr>\n");
   $s .=   "</table>\n";
   return ($s);
}

sub managementUISecurity($$){
   my ($checked, $cgi) = @_;
   my $url_self = $cgi->url(-relative=>1);
   my $s = "";
   $s .= "<h3>Management Interface</h3>\n";
   if (usingNTHost()) {
      $s .= "<p class=\"txt\">You are using Windows NT as a host operating system.  If you would like to " .
            "enable SSL for the Management Interface, please follow the directions detailed in the " .
            "documentation.</p>";
   } else { 
      $s .= ("<table border=0>\n" .
         "<tr>\n" .
         "<td valign=\"top\"><input type=\"checkbox\" name=\"$encryptMUI\" $checked->{$encryptMUI}></td>" .
         "<td class='txt'>Use Secure Sockets Layer (SSL) to encrypt management interface sessions<br>\n" .
         "</td>" .
         "</tr>\n" .
         "</table>\n" 
         );
   }
   return ($s);
}

sub toCustomSecurityHTML($) {
    my ($cgi) = @_;
    my $url_self = $cgi->url(-relative=>1);

    my $s = "";

    $s .= 
        ("<html>\n" .
	 "<script type=\"text/javascript\">\n" .
	 "<!--\n" .
	 "function doConfirmSave() {\n" .
	 "   return confirm(\"Are you sure you want to change the security configuration?\");\n" .
	 "}\n" .
	 "function doSuccess() {\n" .
	 "   return confirm(\"success!\");\n" .
	 "}\n" .
	 "// -->\n" .
	 "</script>\n" .

	 "<form name=\"save\" method=\"POST\" action=\"$url_self\", onSubmit=\"return doConfirmSave()\" onError=\"return doSuccess()\">\n" .
         "\n" );

    my %checked;
    my %options;
    loadSecurityOptions(\%options, "host");

    foreach my $key ($encryptMUI, $encryptRUI) {
       if ($options{$key} =~ m/(Y|T|1)/i) {
         $checked{$key} = "CHECKED";
       } else {
         $checked{$key} = "";
       }
    }

       $s .= "<hr>\n";
       $s .= managementUISecurity(\%checked,$cgi);
    $s .= "<hr>\n";
    $s .= remoteUISecurity(\%checked);

    #XXX ugly
    my $nextUrl = $cgi->url();
    $nextUrl =~ s/$URL_SELF.*$/$URL_INDEX/;  
    $s .=("<br><hr><br>\n" .
	"<table border=0>\n" .
         "<tr>\n" .
         "<td>\n" .
	 "<input type=\"hidden\" name=\"$RCom\" value=\"$RCom_Save\">\n" .
         "<input type=\"submit\" value=\" Save Changes \">\n" .
         "</form>\n".
         "</td>\n" . 
         "<td>\n" .
	 "<form name=\"cancel\" method=\"GET\" action=\"$nextUrl\">\n" .
         "<input type=\"submit\" value=\"Cancel Changes\">\n" .
         "</form>\n" .
         "</td>\n" .
         "</tr>\n" .
         "</table>\n"
	 );
    return($s);
}


#
# loadSecurityOptions
#
# make a serverd request to load all of the security.$subfield entries
# from the host pref database.
#
sub loadSecurityOptions($$) {
   my ($optRef,$subfield) = @_;
   my $in = VMware::DOMAccess->new("exec");
   my $key;
   my $i = 0;

   $in->setValue("exec.op", "Security_GetConfig");
   $in->setValue("exec.in.pattern", "security.$subfield");
   my $out = &VMware::Management::ConfigHTML::request($in);
   if (!$out) {
      Warning("Serverd request failed. Internal error.");
      $in->dispose();
      return;
   }

   # XXX umm.  we lose all relevant error codes here
   while ($key = $out->getValue(".out.pref[$i].name")) {
      $optRef->{$key} = $out->getValue(".out.pref[$i].value");
      $i++;
   } 

   $in->dispose();
   $out->dispose();
}


sub activateChanges($$) {
   my ($optRef, $results) = @_;
   my $in = VMware::DOMAccess->new("exec");
   my $key;
   my $i = 0;

   $in->setValue("exec.op", "Security_ActivateConfig");
   foreach $key (keys %{$optRef}) {
      $in->setValue("exec.in.pref[$i].name", $key);
      $in->setValue("exec.in.pref[$i].value", $optRef->{$key});
      $i++;
   }

   my $out = &VMware::Management::ConfigHTML::request($in);
   if (!$out) {
      Warning("Serverd request failed. Internal error.");
      $in->dispose();
      return 0;
   }

   $i = 0;
   while ($key = $out->getValue(".out.pref[$i].name")) {
      $results->{$key} = $out->getValue(".out.pref[$i].value");
      $i++;
   } 
   return 1;
   $in->dispose();
   $out->dispose();
}

sub handleRequest($) {
    my ($cgi) = @_;
    my $s = "";
    my $url_self = $cgi->url(-relative=>1);
    my $title = "Customize Security Configuration";
    my @path = @htmlPath;

    # XXX Hack
    $gPanicCallback = \&VMware::Management::ConfigHTML::PanicCallbackHTML;


    my $req = $cgi->param($RCom);

    if ( defined($req)) {
       if ($req eq $RCom_Save ) {
          return (saveConfig($cgi));
       }
    }

    $s = &VMware::Management::ConfigHTML::toHTMLPath(\@path, $htmlTitle);
    $s .=  toCustomSecurityHTML($cgi);
	return ($s);
}

#XXX umm. so every module rewrites this function.  should probably be 
# in a util or whatever
my $server = undef;
sub request($) {
    my $doc = shift;
    my $err;
    my %credentials = hash_credentials( );

    my $xml = $doc->getValue("");

    if(!defined($server)) {
	$server = &VMware::Control::Server::new(undef, 0, $credentials{ "l" }, $credentials{ "m" } );

	if (!$server || !$server->connect()) 
	{
	    Warning("Cannot connect", "debug");
	}
    }

    my $ret = $server->exec($xml);
    if (!$ret) {
	my ($err, $errstr) = $server->get_last_error();
	Warning("VMControl error $err: $errstr\n", "error");
    }
	    
    my $resp = VMware::DOMAccess->newXML($ret);

    return $resp;
}

sub handler {
    # Delete the PATH to get by the tainted security requirements of Perl
    delete($ENV{'PATH'}); 

    umask 022;   # Create files that are not writable by group
    
    &clearWarnings();

    my $cgi = new CGI( shift( ) );
    my %MimeHeaders = ( "Content-Type"  => "text/html; charset=iso-8859-1",
			"Pragma"        => "no-cache",
			"Cache-Control" => "no-cache",
			);
    
    my $s = &VMware::Management::ConfigHTML::toHTMLHeader($htmlTitle, $user);

    if (!&VMware::Management::ConfigHTML::isAdminUser()) {
       my $url_self = $cgi->url(-relative=>1);
       $s .= &VMware::Management::ConfigHTML::toErrorHTML("Operation Not Permitted",
			   "<p class=\"txt\">You must have administrative privileges to perform " .
			   "this operation. </p>",
			   $htmlTitle);
    } else {
       $s .= handleRequest($cgi);
    }
    
	$s .= &VMware::Management::ConfigHTML::toHTMLFooter();
    return ( $s, %MimeHeaders );
}

1;
