# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::Login;
# File: VMware/Management/Login.pm

use strict;

use VMware::Control '1.00';
use VMware::Control::Server ();
use VMware::Control::Profiler;
# use VMware::Control::VM ();
use VMware::Management::Util qw( trace
                                 string_to_file
                                 file_to_string
                                 connect_ctrl_obj
                                 append_error
                                 hash_credentials );

use CGI ();
use XML::DOM ();
use XML::Sablotron ();
use MIME::Base64 ();

my $doc;
my $q;
my $p = VMware::Control::Profiler::getInstance("mui");
srand( time( ) ^ ( $$ + ( $$ << 15 ) ) );

sub handler {
    my $stub = $p->profile();
  trace( "handler( ) called.", "debug" );
  $q = new CGI( shift( ) );
  my ( $auth_ok, $e, $s );
  my $k_cookie = "";
  my $s_cookie = "";

#   trace( $q->param( "l" ) .", ". $q->param( "m" ), "debug" );
  if ( $q->param( "l" ) ) {
    trace( "authenticate( ) called.", "debug" );
    ( $auth_ok, $e, $s ) = authenticate( );
    unless ( !$auth_ok ) {
      ( $k_cookie, $s_cookie ) = initialize_session( );

      my $loc = ( $q->param( "n" ) ? MIME::Base64::decode_base64( $q->param( "n" ) ) : $ENV{ "vmware_DEFAULT_URI" } );
      if ($loc eq $ENV{vmware_LOGOUT_URI}) {
         $loc = $ENV{vmware_DEFAULT_URI};
      }
      
      if ( $^O eq "MSWin32" ) {
         return ( qq(<head><META HTTP-EQUIV="Refresh" Content=0;URL="$loc"></head>),
               "Content-type" => "text/html",
               "kid-Cookie" => $k_cookie,
               "sid-Cookie" => $s_cookie );
      
      } else {
         return ( "",
               "Location" => ( $loc ),
               "kid-Cookie" => $k_cookie,
               "sid-Cookie" => $s_cookie );
      }
    }
  }
  my $parser = new XML::DOM::Parser;
  $doc = $parser->parse( "<server></server>" );
  my $root = $doc->getDocumentElement( );
  my $processor = new XML::Sablotron();
  my $xsl = $ENV{ "style_DEFAULT" };
  my $xml = &VMware::Management::Util::tmpdir() . "login.xml";

  &VMware::Management::Util::append_host($doc, $root);

#   my ( $server, $server_ok, $vm, $vm_ok, $err, $errstr );
#   my @vms;
  
#   $server = &VMware::Control::Server::new( undef, 0, undef, undef );
#   ( $server_ok, $err, $errstr ) = connect_ctrl_obj( $server );
#   unless( $server_ok ) {
#     append_error( $doc, $root, $err, $errstr );
#   }

#   @vms = $server->enumerate( );
#   $vm = VMware::Control::VM::new( $server, $vms[0] );
#   ( $vm_ok, $err, $errstr ) = connect_ctrl_obj( $vm );
#   unless( $vm_ok ) {
#     append_error( $doc, $root, $err, $errstr );
#   }

#   if ( $server_ok ) {
#     append_product( $doc, $root, $server );
#   }

  if ( !$auth_ok ) {
    if ( $e ) {
      trace( "$e: $s", "error" );
      append_error( $doc, $root, $e, $s );
      if ( $q->param( "l" ) ) {
        my $username;
        chomp( $username = $q->param( "l" ) );
        &VMware::Management::Util::append_user($doc, $root, $username);
        &VMware::Management::Util::append_build($doc, $root);
      }
    } elsif ($ENV{ "vmware_SESSION_LENGTH" } == 0) {
      append_error($doc, $root, undef, "The VMware Management Interface is closed. No logins permitted.");
    } else  { 
      my %k = hash_credentials( );
      if ( $k{ "l" } ) { # there's a session, but it has expired
        append_error( $doc, $root, undef, "Your session has expired. Please login." );
      }
    }
  }

  if ( $q->param( "n" ) ) {
    my $next;
    chomp( $next = $q->param( "n" ) );
    append_next( $root, $next );
  }

  if ( $ENV{ "vmware_DEBUG_LEVEL" } > 2 ) {
    string_to_file( $doc->toString( ), $xml, 1 );
  }
  
  $processor->SetBase( "file://" . $ENV{ "vmware_BASEHREF" } );
    $p->start("XML::Sablotron::RunProcessor");
  $processor->RunProcessor( "arg:/template", "arg:/data", "arg:/result",
                            undef,
                            [ "template", file_to_string( $xsl ), "data", $doc->toString( ) ] );
    $p->end();
  $doc->dispose( );
  return ( $processor->GetResultArg("result"),
           "Pragma"        => "no-cache",
           "Cache-Control" => "no-cache",
           "Content-Type"  => "text/html; charset=iso-8859-1" );
}

sub authenticate {
    my $stub = $p->profile();
  my $username = MIME::Base64::decode_base64( $q->param( "l" ) );
  my $password = MIME::Base64::decode_base64( $q->param( "m" ) );

  my $server = VMware::Control::Server::new( undef, 0, $username, $password );
  my ( $server_ok, $e, $s ) = connect_ctrl_obj( $server );
  unless ( !$server ) {
    $server->disconnect( );
    trace( "\$server->disconnect( ) called ($server).", "debug" );
  }

  return ( $server_ok, $e, $s );
}

sub initialize_session {
    my $stub = $p->profile();
  my $kc = "vmware.mui.kid=";
  my $sc = undef;
  if ($^O eq "MSWin32") {
    $sc = VMware::Management::Util::validate_session_non_vmdb( $q->param( "l" ) );
  } else {
    $sc = VMware::Management::Session::validate_session( $q->param( "l" ),
                                                         undef,
                                                         $q->param( "m" ));
  }
  # munge username:password:sid
  $kc .= MIME::Base64::encode_base64( $q->param( "l" ).":".$q->param( "m" ).":$sc", "" );
  chomp($kc);
  return ( "$kc;path=/;", "$sc" );
}

sub append_next {
  my ( $root, $next ) = @_;
  $root = $root->appendChild( $doc->createElement( "next" ) );
  $root->appendChild( $doc->createTextNode( $next ) );
}

1;
