# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::MultipartCache;
# File: VMware/Management/MultipartCache.pm
# ---------------------------------------------------------------------------
# Handle anticipated multipart requests (e.g., for FRAMESETs) by generating
# unique files for each part. If a request provides a multipart key, return the
# appropriate file before deleting it.
# ---------------------------------------------------------------------------

use strict;

use VMware::Control::Profiler;
use VMware::Management::Util;
require Exporter;

use vars qw( @ISA @EXPORT );
@ISA = qw( Exporter );
@EXPORT = qw( generate_part retrieve_part scrub_cache );

my $dir = $ENV{ "vmware_MULTIPART_CACHE" } if $ENV{ "MOD_PERL" };
# $seed_min = 15;
# $seed_max = 225;
# $seed = int( rand( $seed_max ) );
# unless( $seed >= $seed_min ) {
#   $seed *= $seed_min;
# }
my $p = VMware::Control::Profiler::getInstance("mui");

sub make_dir {
    my $stub = $p->profile();
  unless ( -e $dir ) {
    trace( "Creating $dir", "notice" );
    mkdir( $dir, 0777 ) or die "Cannot create $dir: $!";
    chmod( 0777, $dir ) or trace( "Cannot chmod $dir: $!", "error" );
  }
}

sub generate_part {
    my $stub = $p->profile();
  my $part = shift( );
    
#   srand( time( ) ^ ( $$ + ( $$ << $seed ) ) );
#   if ( $seed < $seed_max ) {
#     $seed++;
#   } else {
#     $seed = int( rand( $seed_max ) );
#     unless( $seed >= $seed_min ) {
#       $seed *= $seed_min;
#     }
#   }
#   my $key = int( rand( 10000000 ) );
#   while ( -e $dir . ( $key = time( ) . "-" . $key ) ) {
#     trace( "$dir$key already exists!", "debug" );
#     $key = int( rand( 10000000 ) );
#   }

  make_dir( ); # Create directory if it doesn't exist.

  my $sec = time( );
  my $key = `mktemp $dir$sec.XXXXXX`;
  if ( $? ) {
    trace( "Failed to create temporary file: $?", "error" );
  }

  string_to_file( $part, $key );
  trace( "Created $key.", "notice" );
  return $key;
}

# [ ! ] retrieve_part should also remove "expired" files in the unlikely event
# that one or more files get left in this directory.
sub retrieve_part {
    my $stub = $p->profile();
  my $key = shift( );
  my $inc;

  # I'm assuming that if the directory doesn't exist, creating it here won't
  # solve any problems.
  my $part = file_to_string( $key, undef, 1 );

#   $i = unlink( $key );
#   if ( $i == 1 ) {
#     trace( "Removed $key.", "notice" );
#   } else {
#     trace( "Removed $i file(s).", "error" );
#   }

#   foreach my $i ( @INC ) {
#     $inc .= "-I\"$i\" "
#   }
#   chop( $inc );
#   trace( "perl $inc -e 'use VMware::Management::MultipartCache; scrub_cache(\"" . $dir . "\", " . $ENV{ "vmware_RETAIN_PARTS_FOR" } . ");' &", "debug" );
#   system( "perl $inc -e 'use VMware::Management::MultipartCache; scrub_cache(\"" . $dir . "\", " . $ENV{ "vmware_RETAIN_PARTS_FOR" } . ");' &" );
  scrub_cache( $dir, $ENV{ "vmware_RETAIN_PARTS_FOR" } );
  return $part;
}

sub scrub_cache {
    my $stub = $p->profile();
  my $d = shift( );
  my $time_limit = shift( );
  my $time;

  trace( "Looking for $d.", "debug" );
  if ( -e $d ) {
    trace( "Scrubbing $d.", "debug" );
    $time = time( );

    opendir(DIR, $d) || die "can't opendir $d: $!";
    my @files = readdir( DIR );
    my @stale;

    foreach my $file ( @files ) {
      if ( -f ( $d.$file ) ) {
        my $atime = ( stat( $d.$file ) )[ 9 ];
        if ( $time - $atime > 60 * $time_limit ) {
          push( @stale, $d.$file );
          my $i = unlink( $d.$file );
          if ( $i == 1 ) {
            trace( "Removed $d.$file.", "notice" );
          } else {
            trace( "Removed $i file(s).", "error" );
          }
        }
      }
    }

    closedir DIR;
    trace( "Scrubbed $d. Scrubbed " . scalar( @stale ) . " of " .  scalar( @files ) . " files.", "debug" );
  }
}

1;
