# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SS::MemStats;
# File: VMware/Management/SS/MemStats.pm

use strict;

use VMware::HConfig::SSHostRemote;
use VMware::Control '1.00';
use VMware::Control::Server;
use VMware::Control::VM;
use VMware::Control::Profiler;
use VMware::Management::Util qw( trace
                                 file_to_string
                                 string_to_file
                                 hash_credentials
                                 append_error
                                 append_agent
                                 append_newMui
                                 connect_ctrl_obj );

use XML::DOM ();
use XML::Sablotron ();
use CGI ();

my $p = VMware::Control::Profiler::getInstance("mui");

sub handler {
   my $stub = $p->profile();
   my $q = new CGI(shift());
   my %credentials = hash_credentials();
   my ($server, $server_ok, $err, $errstr);
   my $processor = new XML::Sablotron();
   my $xsl = $ENV{ "style_DEFAULT" };
   my $xml;
   my $mui = $q->param("mui");

   $server = &VMware::Control::Server::new(undef, 0,
                                           $credentials{"l"},
                                           $credentials{"m"});
   ($server_ok, $err, $errstr) = connect_ctrl_obj($server);

   if ($server_ok) {
      my $ret = $server->exec("<exec><op>ESXResourceInfo_GetAll</op></exec>");

      if (defined($ret)) {
         $ret = VMware::DOMAccess->newXML($ret);

         # XXX It's possible that serverd should be including this in
         # exec.out.server rather than inserting it here.
         append_agent($ret->getDocument(), $ret->get("exec.out.server"));
         if ($mui) {
            append_newMui($ret->getDocument(), $ret->get("exec.out.server"), $mui);
         }

         # Append system-wide memory stats
         my $mret = $server->exec("<exec><op>ESXHostInfo_GetMemory</op></exec>");
         if (defined($mret)) {
            $mret = VMware::DOMAccess->newXML($mret);
            foreach my $elt ($mret->listElementNames("exec.out")) {
               my $node = $mret->getSubTree("exec.out.$elt");
               $ret->addSubTree("exec.out.server.memory", $node);
               $node->dispose();
            }
            $mret->dispose();
         }

         # Append swap file stats
         my $sret = $server->exec("<exec><op>ESXHostInfo_GetSwapInfo</op></exec>");
         if (defined($sret)) {
            $sret = VMware::DOMAccess->newXML($sret);
            foreach my $elt ($sret->listElementNames("exec.out")) {
               my $node = $sret->getSubTree("exec.out.$elt");
               $ret->addSubTree("exec.out.server.memory.swap", $node);
               $node->dispose();
            }
            $sret->dispose();
         }

         # Compute maximum VM size (for scaling charts) and reserved swap
         # (both done here because they're really awkward to compute in XSLT)
         my $vm_max = 0;
         foreach my $elt ($ret->listElementNames("exec.out.server", "^vm")) {
            my $min = $ret->getValue("exec.out.server.$elt.memory.min");
            my $max = $ret->getValue("exec.out.server.$elt.memory.max");
           
            $vm_max = ($max > $vm_max) ? $max : $vm_max;
         }
         $ret->setValue("exec.out.server.memory.vm_max", $vm_max);
        
         my $swap_tot = $ret->getValue("exec.out.server.memory.swap.total");
         my $swap_unrsvd = $ret->getValue("exec.out.server.memory.swap.unrsvd");
         my $swap_rsvd = $swap_tot - $swap_unrsvd;
         $ret->setValue("exec.out.server.memory.swap.reserved", $swap_rsvd);

         my $sub = $ret->getSubTree("exec.out.server");
         $xml = $sub->toEncodedString('');
         $sub->dispose();
         $ret->dispose();
      }
   } else {
      # rbosch: This is an ESX-only page, make sure we get banner right
      $xml = qq{<server><product id="ESX"/></server>};
      # [ ! ] XXX Trace an error about the inability to connect to vmauthd
      trace("$err: $errstr", "error");
   }

   if ($server_ok) {
      $server->disconnect( );
      trace( "server->disconnect() called ($server).", "notice");
   }

   if ($ENV{ "vmware_DEBUG_LEVEL" } > 2) {
      my $xmlfile;
      if (defined($ENV{"PATH_INFO"}) && $ENV{"PATH_INFO"} ne "/") {
         $_ = $ENV{"PATH_INFO"};
         s/\/(.*?)\/?/$1/g;
         $xmlfile = "/tmp/stats-$_.xml";
      } else {
         $xmlfile = "/tmp/memstats.xml";
      }
      string_to_file($xml, $xmlfile, 1);
   }

   $processor->SetBase("file://" . $ENV{ "vmware_BASEHREF" });
   $p->start("XML::Sablotron::RunProcessor");
   $processor->RunProcessor( "arg:/template", "arg:/data", "arg:/result",
                             undef,
                             [ "template",
                               file_to_string($xsl),
                               "data",
                               $xml ] );
   $p->end();

   return ( $processor->GetResultArg("result"),
            "Pragma"        => "no-cache",
            "Cache-Control" => "no-cache, must-revalidate",
            "Content-Type"  => "text/html; charset=iso-8859-1" );
}

1;
