# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SS::Reboot;
# File: VMware/Management/SS/Reboot.pm

use strict;

use VMware::Control '1.00';
use VMware::Control::Server ();
use VMware::Management::Util qw( trace hash_credentials );

use VMware::HConfig::SSHostRemote;
use VMware::HConfig::Constant qw( $URL_INDEX $URL_OVERVIEW $URL_REBOOT );
use VMware::ExtHelpers qw( &Warning &getWarnings &clearWarnings );

use VMware::Management::SS::ESXConfig;
my $htmlTitle = "System Reboot";

use CGI;

sub handler {
    &clearWarnings();
    
    my $cgi = new CGI( shift( ) );
    my $req = $cgi->param("request");
    my $message = $cgi->param("message");
    
    my %credentials = hash_credentials( );
    my $user = $credentials{ "l" };
    my $password = $credentials{ "m" };

    my $s = toHTMLHeader($htmlTitle, $user) . toHTMLPath(\@htmlPath, $htmlTitle);

    if( $user ne 'root' ) {
	my $url_self = $cgi->url(-relative=>1);
        $s .= toHTMLNotRoot($url_self);
        goto footer;
    }
    
    my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $password);

    if(!defined($host)) {
       $s .= toHTMLHostFailure($htmlTitle);
       goto footer;
    }

    $s .= toHTMLSystemInfo($host) . toHTMLWarnings();

    if ($req eq "reboot") {
       $host->rebootSystem($message);
       # Redirect to prevent inadvertent reboot caused by reloading page
       return("", "Location" => "${URL_REBOOT}?request=reboot-done");
    } elsif ($req eq "halt") {
       $host->haltSystem($message);
       # Redirect to prevent inadvertent halt caused by reloading page
       return("", "Location" => "${URL_REBOOT}?request=halt-done");
    } elsif (($req eq "reboot-done") || ($req eq "halt-done")) {
       $s .= toDoneHTML($cgi, $host, $req);
    } else {
       $s .= toQueryHTML($cgi, $host);
    }

footer:
   $s .= toHTMLFooter();
   return ($s, %MimeHeaders);
}

sub toDoneHTML($$) {
    my ($cgi, $host, $req) = @_;
    my $url_self = $cgi->url(-relative=>1);

    if ($req eq "reboot-done") {
        return <<END_HTML;
<p class='txt'>
Your system is being rebooted. When it starts up again, you may 
return to the <a href='$URL_INDEX'>server configuration menu</a>
or the <a href='$URL_OVERVIEW'>virtual machine overview</a>.
</p>
END_HTML
    } else {
        return <<END_HTML;
<p class='txt'>
Your system is being halted. When you restart the server, you may
return to the <a href='$URL_INDEX'>server configuration menu</a>
or the <a href='$URL_OVERVIEW'>virtual machine overview</a>.
</p>
END_HTML
    }
}

sub toQueryHTML($$) {
    my ($cgi, $host) = @_;
    my $url_self = $cgi->url(-relative=>1);
    my $message = $cgi->param("message");
    $message = "" if !defined($message);

    return <<END_HTML;
<script type="text/javascript">
  <!--
     function doConfirm(op) {
        msg = prompt("Please enter the reason for " + op + "ing the system " +
                     "and click OK, or click Cancel to abort.", "$message");
        if (msg != null) {
           location.href='${url_self}?request='+op+'&message='+escape(msg);
        }
        return false;
     }
  // -->
</script>

<h3>Options:</h3>
<ul>
<li class="txt">
<a href=${url_self}?request=reboot onClick="return doConfirm('reboot')">
  Reboot system
</a>
<li class="txt">
<a href=${url_self}?request=halt onClick="return doConfirm('halt')">
  Halt system
</a>
<li class="txt">
<a href="$URL_INDEX">
  Return to server configuration menu
</a>
</ul>
END_HTML
}


1;
