#!/usr/bin/perl

package VMware::Management::SS::VMConfig;
# File: VMware/Management/SS/VMConfig.pm

use strict;
use VMware::Config;

use VMware::Control '1.00';
use VMware::Control::Server;
use VMware::Control::VM;
use VMware::Control::Profiler;
use VMware::Management::Util;
use VMware::DOMAccess;
use VMware::ExtHelpers qw(&internal_dirname &internal_basename &getFileSeparator);
use VMware::HConfig::CommonConstant qw(@VMX_CFG_EXTENSIONS);
use CGI;
use URI ();

use XML::DOM;
use XML::Sablotron;

use VMware::Management::SS::VMUtil;

my $createdDir;
my $createdCfg;

my $doc;

my $p = VMware::Control::Profiler::getInstance("mui");


sub handler {
    my $stub = $p->profile();

    $doc = VMware::DOMAccess->new("input");

    my $q = new CGI(shift());

    if ( $ENV{ "vmware_DEBUG_LEVEL" } > 2 ) {
      my $cgifile = &VMware::Management::Util::tmpdir() . "vmconfig.cgi";
      my $dump_string = "";
      foreach my $param (sort($q->param())) {
	  $dump_string .= $param . ' = "' . $q->param($param) . "\"\n";
      }
      string_to_file( $dump_string, $cgifile, 1 );
    }

#     LOG("\n-------------------------------------------\n");
#     LOG("Initial doc");
#     LOG("\n-------------------------------------------\n");
#     LOG($doc->prettyPrint() . "\n");
#     LOG("\n-------------------------------------------\n");

    &AppendUserInfo;
    &AppendDefaults;

    my $cfgName = $q->param("c");
    my $cfg = $q->param("fileName");
    # XXX FIX: remove var when config editor/add VM is reformatted to new MUI
    my $ly = $q->param("ly");
    $cfg ||= $cfgName;
    my $registered = 0;
    my $register = 0;
    my %credentials = hash_credentials();

    my $server = &VMware::Control::Server::new(undef,
                                               undef,
                                               $credentials{ "l" },
                                               $credentials{ "m" });
    my ($server_ok, $err, $errstr) = connect_ctrl_obj($server);

    if ($server_ok) {
        $register = $q->param("register") ? 1 : 0;

        for my $file ($server->enumerate()) {
            if ($cfg eq $file) {
                $registered = 1;
                last;
            }
        }
    } else {
        # [ ! ] XXX Return an error about the inability to connect to vmauthd
        trace("$err: $errstr", "error");
        $doc->setValue("input.op", "edit");
        AppendError($errstr);
    }

    if ($q->param("ly") eq "new") {
       $doc->setValue("input.ly", "new");
       trace("\n  New MUI Layout Set for XSL", "debug");
    }

    if ($q->param("formSubmitted") eq "TRUE") {
        if ($q->param("op") ne "new" &&
            ((!$registered && $register) || ($registered && !$register))) {
            my $exec = VMware::DOMAccess->newXML("<exec></exec>");

            if (!$registered && $register) {
                $exec->setValue(".op", "VMRegister");
            }

            if ($registered && !$register) {
                $exec->setValue(".op", "VMUnRegister");
            }

            $exec->setValue(".in.config", $cfg);

            my $ret = VMware::Management::SS::VMUtil::request($exec);
            $exec->dispose();

            $exec = VMware::DOMAccess->newXML($ret);
            unless ($exec->getValue(".returncode") == 1) {
                # error - show the form
                $doc->setValue("input.op", "edit");
                trace($exec->getValue(".messages.msg"), "error");
                AppendError($exec->getValue(".messages.msg"));
            }
        }

	if($q->param("op") eq "edit") {
            my $exec = VMware::DOMAccess->newXML("<exec></exec>");

            $exec->setValue(".op", "ESXVMConfigEdit_EditVM");
            $exec->setValue(".in.file", $q->param("fileName"));
            $exec->setValue(".in.timestamp", $q->param("timeStamp"));

            my $config = new VMware::Config;
            &UpdateConfig($config, $q);

            foreach my $param ($config->list("")) {
              my $d = VMware::DOMAccess->new("param");

              $d->setValue(".name", $param);
              $d->setValue(".value", $config->get($param));

              $exec->addSubTree(".in.params", $d);
              $d->dispose();
            }

            my $ret = VMware::Management::SS::VMUtil::request($exec);
            $exec->dispose();

            $exec = VMware::DOMAccess->newXML($ret);
            if($exec->getValue(".returncode") == 1) {
                # success
                if ($register) {
                   trace("\n  DEBUG: ly for Redirect on success form submit: $ly",
                   "debug");
                   if ($ly eq "new") {
                     return ("<script>window.close();</script>",
                        "Pragma"        => "no-cache",
                        "Cache-Control" => "no-cache",
                        "Content-Type"  => "text/html; charset=iso-8859-1" );
                   } else {
                      return &redirect("/details?c=" .
                      URI::Escape::uri_escape($q->param("fileName"),
                      "^a-zA-Z0-9\\*\\@\\-_\\.\\/"));
                   }
                } else {
                    return &redirect($ENV{vmware_DEFAULT_URI});
                }
            } else {
              # error - show the form
              $doc->setValue("input.op", "edit");
              AppendError($exec->getValue(".messages.msg"));
              &AppendUserSubmittedInfo($q);
              &AppendNetworkingOptions();
              &AppendSystemInfo();
            }

            $exec->dispose();

        } elsif ($q->param("op") eq "edit-raw") {
              my $exec = VMware::DOMAccess->newXML("<exec></exec>");

              $exec->setValue(".op", "ESXVMConfigEdit_EditVM");
              trace($q->param("fileName"), "debug");
              $exec->setValue(".in.file", $q->param("fileName"));
              $exec->setValue(".in.timestamp", $q->param("timeStamp"));
              $exec->setValue(".in.data", $q->param("data"));
              $exec->setValue(".in.o_create", 1);

              my $ret = VMware::Management::SS::VMUtil::request($exec);
              $exec->dispose();

              $exec = VMware::DOMAccess->newXML($ret);
              trace($exec->toEncodedString(), "debug");
              if($exec->getValue(".returncode") == 1) {
                  # success
                  if ($register) {
                     if ($ly eq "new") {
                        return ("<script>window.close();</script>",
                           "Pragma"        => "no-cache",
                           "Cache-Control" => "no-cache",
                           "Content-Type"  => "text/html; charset=iso-8859-1" );
                     } else {
                        return &redirect("/details?c=" .
                        URI::Escape::uri_escape($q->param("fileName"),
                        "^a-zA-Z0-9\\*\\@\\-_\\.\\/"));
                     }
                  } else {
                      return &redirect($ENV{vmware_DEFAULT_URI});
                  }
              } else {
                # error - show the form
                $doc->setValue("input.op", "edit-raw");
                $doc->setValue("input.config.data", $q->param("data"));
                AppendError($exec->getValue(".messages.msg"));
                &AppendUserSubmittedInfo($q);
                &AppendNetworkingOptions();
                &AppendSystemInfo();
              }

              $exec->dispose();

	} else {

            my $exec = VMware::DOMAccess->newXML("<exec></exec>");

            $exec->setValue(".op", "ESXVMConfigEdit_NewVM");

            # trim leading & trailing white space from the file path
            # otherwise vm creation can fail
            my $cfgFl = $q->param("fileName");
            $cfgFl =~ s/^\s+(.*)/$1/g;
            $cfgFl =~ s/(\S+)\s*$/$1/g;
            $exec->setValue(".in.file", $cfgFl);

            my $config = new VMware::Config;
            &UpdateConfig($config, $q);

            foreach my $param ($config->list("")) {
              my $d = VMware::DOMAccess->new("param");

              $d->setValue(".name", $param);
              $d->setValue(".value", $config->get($param));

              $exec->addSubTree(".in.params", $d);
              $d->dispose();
            }

            my $ret = VMware::Management::SS::VMUtil::request($exec);
            $exec->dispose();

            $exec = VMware::DOMAccess->newXML($ret);
            if($exec->getValue(".returncode") == 1) {
              #success
              $doc->setValue("input.success", "TRUE");
              &AppendUserSubmittedInfo($q);
            } else {
              #error - show the form
              $doc->setValue("input.op", "new");
              AppendError($exec->getValue(".messages.msg"));
              &AppendUserSubmittedInfo($q);
              &AppendNetworkingOptions;
              &AppendSystemInfo;
            }

            $exec->dispose();
	}

    } else {
	# show the new/edit form
	&AppendNetworkingOptions;
	&AppendSystemInfo;
	if($cfgName ne "")    {
          # edit/edit-raw
          $doc->setValue("input.op", $q->param("view") eq "file" ? "edit-raw" : "edit");

          my $exec = VMware::DOMAccess->newXML("<exec></exec>");

          $exec->setValue(".op", "ESXVMConfigInfo_GetVM");
          $exec->setValue(".in.file", $cfgName);


          my $ret = VMware::Management::SS::VMUtil::request($exec);
          $exec->dispose();

          $exec = VMware::DOMAccess->newXML($ret);
          if ($exec->getValue(".returncode") == 1) {
            # success

            my $config = new VMware::Config;
            my $params = $exec->getSubTree(".out.params");

            foreach my $param ($params->listElementNames(".")) {
              my $name = $params->getValue(".$param.name");
              my $value = $params->getValue(".$param.value");

              if (defined($name) && defined($value)) {
                $config->set($name, $value);
              }
            }

            # XXX This value may not be accurate if the $server->enumerate
            # operation failed or never happened.
            $doc->setValue("input.config.registered", $registered);

            $doc->setValue("input.config.timeStamp",
                           $exec->getValue(".out.timestamp"));
            $exec->dispose();

            AppendVMInfoConfig($cfgName, $config);

            if ($q->param("view") eq "file") {

              $exec = VMware::DOMAccess->newXML("<exec></exec>");

              $exec->setValue(".op", "File_Read");
              $exec->setValue(".in.file", $cfgName);


              $ret = VMware::Management::SS::VMUtil::request($exec);
              $exec->dispose();

              $exec = VMware::DOMAccess->newXML($ret);
              if($exec->getValue(".returncode") == 1) {
                # success
                my $config = new VMware::Config($cfgName);

                $doc->setValue("input.config.data",
                             $exec->getValue(".out.data"));
                $exec->dispose();
              } else {
                # error - show the form
                # XXX There's no error message.
                AppendError($exec->getValue(".messages.msg"));
              }
            }
          } else {
            # error - show the form
            # XXX There's no error message.
            AppendError($exec->getValue(".messages.msg"));
          }

          $exec->dispose();
	} else {
          # new
          $doc->setValue("input.op", "new");
          &AppendWizardInfo;
	}
    }

    my $xml = $doc->toEncodedString("");

    $doc->dispose( );
    return &emit_xml($xml);
}


###
### query -> config
###
sub UpdateConfig {
    my $stub = $p->profile();
    my $config = shift();
    my $q = shift();
    my $os = $q->param("os");

    # Retrieve the unmodified config parameters first since we will be doing
    # processing on the config file paramters and we want these parameters in
    # the config object when we perform our processing.
    my @p = split(/=/, $q->param("other"));
    my($c, $v);
    while (($c, $v) = splice(@p, 0, 2)) {
      $config->set($c, $v);
    }

    # in other $config->set("config.version", $q->param("configVersion"));
    $config->set("guestOS", $q->param("os"));
    $config->set("displayName", $q->param("displayName"));
    $config->set("memsize", $q->param("memSize"));
    $config->set("numvcpus", $q->param("numVCPUs"));

    my $present        = $q->param("cdrom") ? "TRUE" : "FALSE";
    my $startConnected = $q->param("cdromStartConnected") ? "TRUE" : "FALSE";
    my $deviceType     = $q->param("cdromDeviceType") ? "cdrom-image" : "atapi-cdrom";
    $config->set("ide1:0.present", $present);
    $config->set("ide1:0.fileName", $q->param("cdromFileName"));
    $config->set("ide1:0.deviceType", $deviceType);
    $config->set("ide1:0.startConnected", $startConnected);

    $present        = $q->param("floppy0") ? "TRUE" : "FALSE";
    $startConnected = $q->param("floppy0StartConnected") ? "TRUE" : "FALSE";
    $deviceType     = $q->param("floppy0FileType") ? "file" : "device";
    $config->set("floppy0.present", $present);
    $config->set("floppy0.fileName", $q->param("floppy0FileName"));
    $config->set("floppy0.fileType", $deviceType);
    $config->set("floppy0.startConnected", $startConnected);

    foreach my $j (0..1) {
      my $adapterPresent = ($j == 0) ? "TRUE" : "FALSE";

      foreach my $i (1..16) {
        $present = $q->param("scsi${j}_${i}_present");
        if ($present) {
          $adapterPresent = "TRUE";
        }
        if (defined($present)) {
          my $scsiVol = $q->param("scsi${j}_${i}_volume");
          my $scsiName = $q->param("scsi${j}_${i}_fileName");
          $config->set("scsi${j}:${i}.present", ($present) ? "TRUE" : "FALSE");
          $config->set("scsi${j}:${i}.name", $scsiVol . ":" . $scsiName);
          my $mode = $q->param("scsi${j}_${i}_mode");
          if( defined($mode) && $mode ne "" ) {
            $config->set("scsi${j}:${i}.mode", $mode);
          }
          my $size = $q->param("scsi${j}_${i}_size");
          if( defined($size) && $size ne "" ) {
            $config->set("scsi${j}:${i}.size", $size);
          }
        }
      }

      if ($adapterPresent eq "TRUE") {
        $config->set("scsi${j}.present", $adapterPresent);
        my $virtualDev = $q->param("scsi${j}_virtualDev");
        # Assign SCSI driver if its not assigned or is one of the predefined
        # drivers. Custom defined drivers will not be auto-set and will
        # remain the same.
        if ($virtualDev eq "" || $virtualDev eq "vmxbuslogic" ||
        $virtualDev eq "vmxlsilogic") {
           # force old SCSI driver for follow OSs; else use new driver
           if ($os eq "linux" || $os eq "nt4" || $os eq "win2000" ||
           $os eq "winXPPro") {
              $virtualDev = "vmxbuslogic";
           } else {
              $virtualDev = "vmxlsilogic";
           }
           trace("\n  Set SCSI Driver based on OS: " . $q->param("os"), "debug");
           trace("\n  Driver Being Used: " . $virtualDev, "debug");
        }
        $config->set("scsi${j}.virtualDev", $virtualDev);
        my $shared = $q->param("scsi${j}_shared");
        if (defined($shared)) {
          $config->set("scsi${j}.sharedBus", $shared);
        }
      }
    }

    foreach my $i (0..3) {
      my $p = $q->param("eth${i}");
      if (!defined($p) || ($p eq "none")) {
        $config->remove("ethernet${i}.present");
        $config->remove("ethernet${i}.virtualDev");
        $config->remove("ethernet${i}.connectionType");
        $config->remove("ethernet${i}.devName");
        $config->remove("ethernet${i}.devName");
      } else {
	$config->set("ethernet${i}.present", "TRUE");
	$config->set("ethernet${i}.virtualDev", $q->param("eth${i}.virtualDev"));

        if ($p eq "bridged") {
          $config->set("ethernet${i}.connectionType", "bridged");
        } elsif ($p eq "hostOnly") {
          $config->set("ethernet${i}.connectionType", "hostOnly");
        } else {
          $config->set("ethernet${i}.connectionType", "monitor_dev");
          $config->set("ethernet${i}.devName", $p);
        }
      }
    }

    $config->set("RemoteDisplay.depth", $q->param("remoteDisplay"));
    if ($q->param("redoLogDir")) {
        $config->set("redoLogDir", $q->param("redoLogDir"));
    }
    my $suspendLocation = $q->param("suspendLocation");
    if ($suspendLocation eq "configdir") {
    # Config directory is default suspend location -- no need to set value
    } elsif ($suspendLocation eq "vmfs") {
        my $volume = $q->param("suspendVolume");
        if ($volume) {
            $config->set("suspend.directory", "/vmfs/$volume");
        }
    } elsif ($suspendLocation eq "other") {
        my $directory = $q->param("suspendDirectory");
        if ($directory) {
            $config->set("suspend.directory", $directory);
        }
    } else {
        # Nothing selected; store in default location
    }

    my $suspendName = $q->param("suspendName");
    if (($suspendLocation ne "configdir") && !$suspendName) {
        # rbosch: If we're storing suspended state files outside of the
        # config directory (e.g., on a VMFS volume), we may encounter
        # name conflicts.  To make this less likely, include a hash of
        # the config file directory in the suspend file name.
        my $configFile = $q->param("fileName");

        # Suspend name base: config filename without extension

        my $configBase = internal_basename($configFile);
        foreach my $ext (@VMX_CFG_EXTENSIONS) {
            if ($configBase =~ /\.$ext$/) {
                $configBase =~ s/\.$ext$//;
                last;
            }
        }

        # Suspend name suffix: hash of config pathname (djb2 hash algorithm)

        # Add some randomness to avoid conflicts in case user moves
        # or deletes this VM and creates a new one in the same place
        my $random = int(128 * rand());
        # prevent perl from converting integer to float in this block
        use integer;
        my $hash = 5381;
        my $configPath = internal_dirname($configFile);
        my @digits = map {ord($_)} split(/ */, $configPath);
        unshift @digits, $random;
        foreach my $d (@digits) {
            $hash = $hash * 33 + $d;
        }

        $suspendName = sprintf("%s-%08x", $configBase, $hash);
    }

    if ($suspendName) {
        $config->set("checkpoint.cptConfigName", $suspendName);
    }

    if( $q->param("debugLevel") eq "TRUE" ) {
	# The default behavior in release build is to have debug off.  The default
	# behavior in a beta build is to have debug on.  Let defaults reign unless
	# the user explicitly turns on debugging.
	# XXX This option will be meaningless in debug builds, but is correct for
   # release builds. -jhu
	$config->set("debug", "TRUE");
    }
}


sub AppendUserInfo {
    my $stub = $p->profile();
    my %credentials = hash_credentials( );
    my $owner = $credentials{ "l" };
    my $home = (getpwnam($credentials{ "l" }))[7];
    my $host = `hostname -s`;
    chomp $host;
    my $domain = `hostname -d`;
    chomp $domain;

    $doc->setValue("input.username", $owner);
    $doc->setValue("input.homeDir", $home);
    $doc->setValue("input.host", $host . "." . $domain);
}


sub AppendDefaults {
    my $stub = $p->profile();
    # my $hostRAM = &GetHostRAM;
    my $hostRAM = 0;

    $doc->setValue("input.defaults.ram.dos", 16);
    $doc->setValue("input.defaults.ram.win31", 16);
    $doc->setValue("input.defaults.ram.win95", 64);
    $doc->setValue("input.defaults.ram.win98", 64);

    my $defaultRAM;

    if($hostRAM < 512) {
        $defaultRAM = 128;
    } else {
        $defaultRAM = 256;
    }

    $doc->setValue("input.defaults.ram.default", $defaultRAM);
}


sub AppendWizardInfo {
    my $stub = $p->profile();
    my %credentials = hash_credentials( );
    my $owner = $credentials{ "l" };
    my $home = (getpwnam($credentials{ "l" }))[7];

    $doc->setValue("input.config.os", "win2000");
    $doc->setValue("input.config.displayName", "My Windows 2000 VM");

    $doc->setValue("input.config.fileName", $home . "/vmware/win2000/win2000.cfg");
    $doc->setValue("input.config.safe_fileName", URI::Escape::uri_escape($home . "/vmware/win2000/win2000.cfg", "^a-zA-Z0-9\\*\\@\\-_\\.\\/"));
    $doc->setValue("input.config.memSize", 128);
    $doc->setValue("input.config.numVCPUs", 1);

    $doc->setValue("input.config.ide10.present", "TRUE");
    $doc->setValue("input.config.ide10.fileName", "/dev/cdrom");
    $doc->setValue("input.config.ide10.deviceType", "atapi-cdrom");
    $doc->setValue("input.config.ide10.startConnected", "TRUE");

    $doc->setValue("input.config.scsi.id", "scsi0");
    $doc->setValue("input.config.scsi.label", "scsi0");
    $doc->setValue("input.config.scsi.shared", "none");

    $doc->setValue("input.config.scsi.disk.id", "scsi0_1");
    $doc->setValue("input.config.scsi.disk.label", "scsi0:1");
    $doc->setValue("input.config.scsi.disk.present", "TRUE");
    $doc->setValue("input.config.scsi.disk.volume", "");
    $doc->setValue("input.config.scsi.disk.fileName", "Untitled.dsk");
    $doc->setValue("input.config.scsi.disk.size", "4096");

    $doc->setValue("input.config.ethernet.adapter.id", "eth0");
    $doc->setValue("input.config.ethernet.adapter.label", "Ethernet Adapter 0");
    $doc->setValue("input.config.ethernet.adapter.devName", "");

    $doc->setValue("input.config.floppy0.present", "TRUE");
    $doc->setValue("input.config.floppy0.fileName", "/dev/fd0");
    $doc->setValue("input.config.floppy0.fileType", "device");
    $doc->setValue("input.config.floppy0.startConnected", "FALSE");

    $doc->setValue("input.config.other", "config.version=6=ethernet0.virtualDev=vmxnet=");
}


sub AppendUserSubmittedInfo {
    my $stub = $p->profile();
    my $q = shift();

    if (defined($q->param("register"))) {
       $doc->setValue("input.config.registered", $q->param("register"));
    }

    if( defined($q->param("timeStamp")) ) {
       $doc->setValue("input.config.timeStamp", $q->param("timeStamp"));
    }

    $doc->setValue("input.config.os", $q->param("os"));
    $doc->setValue("input.config.displayName", $q->param("displayName"));
    $doc->setValue("input.config.fileName", $q->param("fileName"));
    $doc->setValue("input.config.safe_fileName", URI::Escape::uri_escape($q->param("fileName"), "^a-zA-Z0-9\\*\\@\\-_\\.\\/"));
    $doc->setValue("input.config.memSize", $q->param("memSize"));
    $doc->setValue("input.config.numVCPUs", $q->param("numVCPUs"));

    $doc->setValue("input.config.ide10.present", $q->param("cdrom"));
    $doc->setValue("input.config.ide10.fileName", $q->param("cdromFileName"));
    my $deviceType = $q->param("cdromDeviceType") ? "cdrom-image" : "atapi-cdrom";
    $doc->setValue("input.config.ide10.deviceType", $deviceType);
    $doc->setValue("input.config.ide10.startConnected", $q->param("cdromStartConnected"));

    $doc->setValue("input.config.floppy0.present", $q->param("floppy0"));
    $doc->setValue("input.config.floppy0.fileName", $q->param("floppy0FileName"));
    $deviceType = $q->param("floppy0FileType") ? "file" : "device";
    $doc->setValue("input.config.floppy0.fileType", $deviceType);
    $doc->setValue("input.config.floppy0.startConnected", $q->param("floppy0StartConnected"));

    foreach my $j (0..1) {
      $doc->setValue("input.config.scsi[$j].id", "scsi${j}");
      $doc->setValue("input.config.scsi[$j].label", "scsi${j}");
      $doc->setValue("input.config.scsi[$j].shared", $q->param("scsi${j}_shared"));
      $doc->setValue("input.config.scsi[$j].virtualDev", $q->param("scsi${j}_virtualDev"));

      my $indx = 0;

      foreach my $i (1..16) {
        my $present = $q->param("scsi${j}_${i}_present");
        if (defined($present)) {
          $doc->setValue("input.config.scsi[$j].disk[$indx].id", "scsi${j}_${i}");
          my $d = $doc->getSubTree("input.config.scsi[$j].disk[$indx]");
          $indx++;
          $d->setValue(".label", "scsi${j}:${i}");
          $d->setValue(".present", ($present) ? "TRUE" : "FALSE");
          $d->setValue(".volume", $q->param("scsi${j}_${i}_volume"));
          $d->setValue(".fileName", $q->param("scsi${j}_${i}_fileName"));
          my $mode = $q->param("scsi${j}_${i}_mode");
          if( defined($mode) && $mode ne "" ) {
            $d->setValue(".mode", $mode);
          }
          my $size = $q->param("scsi${j}_${i}_size");
          if( defined($size) && $size ne "" ) {
            $d->setValue(".size", $size);
          }
          $d->setValue(".volume", $q->param("scsi${j}_${i}_volume"));
        }
      }
    }

    my $indx = 0;

    foreach my $i (0..3) {
      my $p = $q->param("eth${i}");
      if (defined($p)) {
        $doc->setValue("input.config.ethernet.adapter[$indx].id", "eth${i}");
        $doc->setValue("input.config.ethernet.adapter[$indx].label",
                       "Ethernet Adapter ${i}");
        $doc->setValue("input.config.ethernet.adapter[$indx].devName", $p);
        $doc->setValue("input.config.ethernet.adapter[$indx].virtualDev", $q->param("eth${i}.virtualDev"));
        $indx++;
      }
    }

    $doc->setValue("input.config.remoteDisplay", $q->param("remoteDisplay"));
    if ($q->param("redoLogDir")) {
        $doc->setValue("input.config.redoLogDir", $q->param("redoLogDir"));
    }
    if ($q->param("suspendDirectory")) {
        $doc->setValue("input.config.suspend.directory", $q->param("suspendDirectory"));
    }
    if ($q->param("suspendName")) {
        $doc->setValue("input.config.suspend.name", $q->param("suspendName"));
    }
    $doc->setValue("input.config.debugLevel", $q->param("debugLevel"));

    $doc->setValue("input.config.other", $q->param("other"));
}


sub emit_xml {
    my $stub = $p->profile();
    my $xml = shift();
    my $xsl = $ENV{ "style_DEFAULT" };

    my $parser = new XML::DOM::Parser;
    my $xmldoc = $parser->parse($xml);

    my $processor = new XML::Sablotron();

    if ( $ENV{ "vmware_DEBUG_LEVEL" } > 2 ) {
      my $xmlfile = &VMware::Management::Util::tmpdir() . "vmconfig.xml";
      string_to_file( $xml, $xmlfile, 1 );
    }

    $processor->SetBase( "file://" . $ENV{ "vmware_BASEHREF" } );
    $p->start("XML::Sablotron::RunProcessor");
    $processor->RunProcessor( "arg:/template", "arg:/data", "arg:/result",
			      undef,
			      [ "template", file_to_string( $xsl ), "data", $xmldoc->toString( ) ] );
    $p->end();

    return ( $processor->GetResultArg("result"),
   	     "Content-Type"  => "text/html; charset=iso-8859-1",
           "Cache-Control" => "no-cache, must-revalidate",
           "Pragma"        => "no-cache" );

}


sub redirect {
    my $url = shift();

    trace("Redirecting to $url", "debug");
    return ("",
            "Location"  => $url);
}


sub AppendVMInfo {
    my $stub = $p->profile();
    my $ret;
    my $cfgFile = shift();
    my $text = "";

    open (CFG_FILE, "$cfgFile");
    while (<CFG_FILE>) {
        $text = $text . $_;
    }

    my $config = new VMware::Config;

    $ret = $config->parse($text);
    if (!$ret) {
        trace("Can't read config info:\n$text", "error");
        Cleanup();
        return;
    }

    AppendVMInfoConfig($cfgFile, $config);
}


sub AppendVMInfoConfig {
    my $stub = $p->profile();
    my ($cfgFile, $config) = @_;

    $config->unmark_all();

    $doc->setValue("input.config.fileName", $cfgFile);
    $doc->setValue("input.config.safe_fileName", URI::Escape::uri_escape($cfgFile, "^a-zA-Z0-9\\*\\@\\-_\\.\\/"));
    $doc->setValue("input.config.os", $config->get("guestOS"));
    $doc->setValue("input.config.displayName", $config->get("displayName"));

#    $doc->setValue("input.config.cpuShares", $config->get("sched.cpu.shares",
#                                                          1000));
#    $doc->setValue("input.config.cpuAffinity", $config->get("sched.cpu.affinity",
#                                                            "all"));


    my $memsize = $config->get("memSize");
    $doc->setValue("input.config.memSize", $config->get("memSize"));
#    $doc->setValue("input.config.minMemSize", $config->get("sched.mem.minsize",
#                                                           $memsize/2));
#    $doc->setValue("input.config.memShares", $config->get("sched.mem.shares",
#                                                          1000));

    my $numvcpus = $config->get("numvcpus");
    $doc->setValue("input.config.numVCPUs", $numvcpus);

    foreach my $j (0..1) {
      $doc->setValue("input.config.scsi[$j].id", "scsi${j}");
      $doc->setValue("input.config.scsi[$j].label", "scsi${j}");
      my $sharedBus = $config->get("scsi${j}.sharedBus");
      if (defined($sharedBus)) {
        $doc->setValue("input.config.scsi[$j].shared", $sharedBus);
      } else {
        $doc->setValue("input.config.scsi[$j].shared", "none");
      }

      my $virtualDev = $config->get("scsi${j}.virtualDev");
      trace("\n  SCSI Device Driver: $virtualDev", "debug");
      $doc->setValue("input.config.scsi[$j].virtualDev", $virtualDev);

      # Presence of adapter is inferred from presence of disks --
      # read this parameter so it isn't lumped into 'other'.
      $config->get("scsi${j}.present");

      my $indx = 0;

      foreach my $i (1..16) {
        my $present = $config->get_bool("scsi${j}:${i}.present", 0);
        if ($present || ($i <= 6)) {
          my $disk = $config->get("scsi${j}:${i}.name");
          my @parts = split(/:/, $disk);
          my $diskname = pop(@parts);
          if( $diskname =~ /^[0-9]+$/ ) {
            push(@parts, $diskname);
            $diskname = "";
          }
          my $diskvol = join(":", @parts);

          $doc->setValue("input.config.scsi[$j].disk[$indx].present",
                         ($present) ? "TRUE" : "FALSE");
          my $d = $doc->getSubTree("input.config.scsi[$j].disk[$indx]");
          $indx++;

          $d->setValue(".id", "scsi${j}_${i}");
          $d->setValue(".label", "scsi${j}:${i}");
          $d->setValue(".volume", $diskvol);
          $d->setValue(".fileName", $diskname);
          my $mode = $config->get("scsi${j}:${i}.mode");
          if( $mode ne "" ) {
             $d->setValue(".mode", $mode);
          }
          my $size = $config->get("scsi${j}:${i}.size");
          if( defined($size) && $size ne "" ) {
            $d->setValue(".size", $size);
          }
        }
      }
    }

    my $indx = 0;

    foreach my $i (0..3) {
      my $present = $config->get_bool("ethernet${i}.present", 0);
      if ($present || ($i <= 2)) {
        $doc->setValue("input.config.ethernet.adapter[$indx].id", "eth${i}");
        $doc->setValue("input.config.ethernet.adapter[$indx].label", "Ethernet Adapter ${i}");
        if (!$present) {
          $doc->setValue("input.config.ethernet.adapter[$indx].devName", "none");
        } else {
          my $cnx = $config->get("ethernet${i}.connectionType", "monitor_dev");
          if ($cnx =~ /^monitor_dev$/i) {
            $doc->setValue("input.config.ethernet.adapter[$indx].devName",
                           $config->get("ethernet${i}.devName"));
          } else {
            $cnx =~ tr/A-Z/a-z/;
            $doc->setValue("input.config.ethernet.adapter[$indx].devName", $cnx);
          }
          $doc->setValue("input.config.ethernet.adapter[$indx].virtualDev", $config->get("ethernet${i}.virtualDev"));
        }
        $indx++;
      }
    }

    $doc->setValue("input.config.ide10.present", $config->get("ide1:0.present"));
    $doc->setValue("input.config.ide10.fileName", $config->get("ide1:0.fileName"));
    $doc->setValue("input.config.ide10.deviceType", $config->get("ide1:0.deviceType"));
    $doc->setValue("input.config.ide10.startConnected", $config->get("ide1:0.startConnected"));

    $doc->setValue("input.config.floppy0.present", $config->get("floppy0.present"));
    $doc->setValue("input.config.floppy0.fileName", $config->get("floppy0.fileName"));
    $doc->setValue("input.config.floppy0.fileType", $config->get("floppy0.fileType"));
    $doc->setValue("input.config.floppy0.startConnected", $config->get("floppy0.startConnected"));

    $doc->setValue("input.config.remoteDisplay", $config->get("RemoteDisplay.depth"));
    $doc->setValue("input.config.suspend.directory", $config->get("suspend.directory"));
    $doc->setValue("input.config.suspend.name", $config->get("checkpoint.cptConfigName"));
    $doc->setValue("input.config.redoLogDir", $config->get("redoLogDir"));
    $doc->setValue("input.config.debugLevel", $config->get_bool("debug", 0) ? "TRUE" : "FALSE");

    my $t = "";

    foreach my $c ($config->get_unmarked()) {
      my $v = $config->get($c);
      $t = "$c=$v=$t";
    }

    $doc->setValue("input.config.other", $t);
}


sub AppendError {
    my $err = shift();
    trace("**************** APPENDING ERROR *********************\n", "debug");
    $doc->setValue("input.error", $err);
}


sub AppendNetworkingOptions {
    my $stub = $p->profile();

    my $subTree = VMware::Management::SS::VMUtil::GetVmkNetAdapters();
    $doc->addSubTree("input", $subTree);
    $subTree->dispose();

    $subTree = VMware::Management::SS::VMUtil::GetVmkNetBondAdapters();
    $doc->addSubTree("input", $subTree);
    $subTree->dispose();

    foreach my $i (0..7) {
      my $t = VMware::DOMAccess->new("vmkNet");
      $t->setValue("vmkNet.name", "vmnet_$i");
      $doc->addSubTree("input.netAdapters", $t);
      $t->dispose();
    }
}


sub AppendSystemInfo {
    my $stub = $p->profile();
    # Note that the permissions argument passed to GetVMFS should
    # exactly match the DISK_PERMISSION_FILTER in vmconfig.xsl
    my $mfsTree = VMware::Management::SS::VMUtil::GetVMFS("r");
    $doc->addSubTree("input", $mfsTree);
    $mfsTree->dispose();

    my $ptTree = VMware::Management::SS::VMUtil::GetVmkScsiPassThrough();
    $doc->addSubTree("input", $ptTree);
    $ptTree->dispose();

    my $memTree = VMware::Management::SS::VMUtil::GetMemory();
    $doc->addSubTree("input", $memTree);
    $memTree->dispose();

    my $licTree = VMware::Management::SS::VMUtil::GetLicense();
    $doc->addSubTree("input", $licTree);
    $licTree->dispose();
}


sub LOG {
    my($msg) = @_;
    print STDERR "$msg";
}


sub Cleanup {
    my $stub = $p->profile();
    my $err = shift;

    $doc->setValue("input.error", $err);

    if (defined($createdCfg)) {
	unlink($createdCfg);
    }

    if (defined($createdDir)) {
	rmdir($createdDir);
    }
}

1;
