#!/usr/bin/perl

package VMware::Management::SS::VMKernelConfig;
# File: VMware/Management/SS/VMKernelConfig.pm

use strict;
use VMware::Config;

use VMware::Control '1.00';
use VMware::Control::Server;
use VMware::Management::Util;
use VMware::DOMAccess;
use CGI;
use URI ();

use XML::DOM;
use XML::Sablotron;

my $q;
my $mui; # will be set as 'new' if the new MUI is being used

sub handler {
  $q = new CGI(shift());
  my %credentials = hash_credentials();
  my $parser = new XML::DOM::Parser;
  my $version = VMware::Control::version();
  my $processor = new XML::Sablotron();
  my $want_version = "1.00";
  my $xsl = $ENV{ "style_DEFAULT" };
  my $xml;
  my $root;
  my $doc;

  my $server = &VMware::Control::Server::new(undef, 0,
                                             $credentials{"l"},
                                             $credentials{"m"});

  if ( defined( $server ) && $server->connect( ) ) {
    trace( "Connected to server object ($server).", "notice" );
    my $ret = $server->exec("<exec><op>ESXVMKernelInfo_GetAll</op><in></in></exec>");

    # If $server->exec failed to return any value, don't enter the following
    # block. $xml will not be defined and the block of code after this one
    # will try to contact each vmx process in turn to ascertain the vm state
    # and the state of the system. If $ret is defined, assume that it is
    # valid XML.
    if (defined($ret)) {
      $ret = VMware::DOMAccess->newXML($ret);
      $root = $ret->get("exec.out.server");
      $doc = $ret->getDocument();

      if (!defined($root)) {
        $ret->setValue("exec.out.server", "");
        $root = $ret->get("exec.out.server");
      }

      # variable used to hide content not necessary within the new MUI frame
      $mui = $q->param("mui");
      if ($mui eq "new") {
         trace("New MUI set.", "debug");
         $ret->setValue("exec.out.server.mui", "new");
      }

      if ($ret->getValue("exec.returncode") != 1) {
        foreach my $i ($ret->listElementNames("exec.messages")) {
          &VMware::Management::Util::append_error($doc, $root, undef,
                                                  $ret->getValue("exec.messages.$i"));
        }
      }

      &VMware::Management::Util::append_host($doc, $root);
      &VMware::Management::Util::append_product($doc, $root, $server);
      &VMware::Management::Util::append_agent($doc, $root);
      &VMware::Management::Util::append_user($doc, $root, $credentials{"l"});
      &VMware::Management::Util::append_build($doc, $root );

      $server->disconnect( );
      trace( "server->disconnect( ) called ($server).", "notice" );

      my $sub = $ret->getSubTree("exec.out.server");
      $xml = $sub->getValue('');
      $sub->dispose( );
      $ret->dispose( );
    }
  }

  # $xml is not definied for some reason (maybe $server->exec failed). Try to
  # contact each vmx process to ascertain the vm state and the state of the
  # system.
  unless(defined($xml)) {
    $doc = $parser->parse( "<server></server>" );
    $root = $doc->getDocumentElement( );
    my @vms;
    my $vm;

    &VMware::Management::Util::append_error( $doc, $root, -9999, "Virtual machine control API version mismatch. Want: $want_version. Have: $version." )
      unless $want_version eq $version;

    &VMware::Management::Util::append_host( $doc, $root );
    &VMware::Management::Util::append_agent( $doc, $root );
    &VMware::Management::Util::append_user( $doc, $root, $credentials{ "l" } );
    &VMware::Management::Util::append_build( $doc, $root );

    if ( defined( $server ) && $server->connect( ) ) {
      trace( "Connected to server object ($server).", "notice" );
    } else {
      # &! XXX Return an error about the inability to connect to vmauthd
      my ( $err, $errstr ) = $server->get_last_error( );
      trace( "Could not connect to vmware-authd", "error" );
      trace( $errstr, "error" );
      &VMware::Management::Util::append_error( $doc, $root, $err, $errstr );
    }

    if ( $server && $server->is_connected() ) {
      $server->disconnect( );
      trace( "server->disconnect( ) called ($server).", "notice" );
    }

    $xml = $doc->toString( );
    $doc->dispose( );
  }

  $processor->SetBase("file://" . $ENV{ "vmware_BASEHREF" });
  $processor->RunProcessor( "arg:/template", "arg:/data", "arg:/result",
                            undef,
                            [ "template",
                              file_to_string($xsl),
                              "data",
                              $xml ] );

  return ( $processor->GetResultArg("result"),
           "Pragma"        => "no-cache",
           "Cache-Control" => "no-cache, must-revalidate",
           "Content-Type"  => "text/html; charset=iso-8859-1" );
}

1;
