#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# ViewLog.pm
# 
# A script that allows users to view the system logs
package VMware::Management::SS::ViewLog;

use strict;

use VMware::HConfig::SSHostRemote;
use VMware::ExtHelpers qw( &Warning &clearWarnings );
use VMware::Management::Util qw( &hash_credentials );
use VMware::Management::SS::VMUtil;

use VMware::Management::SS::ESXConfig;
my $htmlTitle = "Log File Viewer";

use CGI;
use Carp qw(confess);
$SIG{__DIE__} = \&confess;

my $logDir = "/var/log";

my %logInfo = (
  "messages" => {
                 "summary" => "All Console OS messages", 
                },
  "vmkernel" => {
                 "summary" => "All VMkernel messages",
                },
  "vmkwarning" => {
                    "summary" => "VMkernel warnings and system alerts",
                    "description" => 
                       "<p class=\"txt\">If you see any lines that begin with ALERT, " .
                       "check the VMware Knowledge Base or contact your " . 
                       "support representative for information on how to " .
                       "correct the problem.</p><hr>\n", 
                  }
);


sub toViewLogHTML($$) {
   my ($cgi, $host) = @_;
   my $log = $cgi->param("log");
   my $url_self = $cgi->url(-relative=>1);

   my $s = <<END_HTML;
<h3>$htmlTitle</h3>

<p class="txt">
This page allows you to view the contents of the system log files.
Select one of the following log files to view:
</p>

<form method="get" action="$url_self">
<select name="log">
END_HTML

   foreach (sort keys %logInfo) {
      my $selected = ($_ eq $log) ? "selected" : "";
      $s .= qq(<option value="$_" $selected>$logInfo{$_}{summary} ($logDir/$_)\n);
   }

   $s .= <<END_HTML;
</select>
<input type="submit" value="View Log">
</form>
END_HTML

   if ($log ne "") {
      my $filename = "$logDir/$log";
      my ($result, $contents) = VMware::Management::SS::VMUtil::ReadFile($filename);
      if ($result == 1) {
         my $header = "$logInfo{$log}{summary}" || "Contents of $filename";
         $s .= "<hr>\n<h4>$header</h4><pre>\n";
         $s .= $logInfo{$log}{description};
         foreach (split /\n/, $contents) {
            if (m/ALERT/) {
               $s .= "<b><FONT COLOR=\"#FF0000\">$_\n</FONT></b>"
            } else {
               $s .= $_ . "\n";
            }
         }
         $s .= "</pre>\n"
      } else {
         $s .= qq(<hr><p class="txt">Cannot display $filename: $contents.</p>\n);
      }
   }

   return($s);
}

sub handler {
   &clearWarnings();
   
   my $cgi = new CGI( shift( ) );
   my %credentials = hash_credentials( );
   
   my $user = $credentials{ "l" };
   my $password = $credentials{ "m" };

   my $s = toHTMLHeader($htmlTitle, $user) . toHTMLPath(\@htmlPath, $htmlTitle);
   
   if ($user ne 'root') {
      my $url_self = $cgi->url(-relative=>1);
      $s .= toHTMLNotRoot($url_self);
      goto footer;
   }

   my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $password);
   if (!defined($host)) {
      $s .= toHTMLHostFailure($htmlTitle);
      goto footer;
   }

   $s .=
       toHTMLSystemInfo($host) .
       toHTMLWarnings() .
       toViewLogHTML($cgi, $host);

footer:
   $s .= toHTMLFooter();
   return ($s, %MimeHeaders);
}

1;
