# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::ShowDirTree;
# File: VMware/Management/ShowDirTree.pm

use strict;

use VMware::Control '1.00';
use VMware::Control::Server;
use VMware::Control::VM;
use VMware::Management::Util qw( trace
                                 file_to_string
                                 string_to_file
                                 hash_credentials
                                 append_error
                                 connect_ctrl_obj );

use XML::DOM ();
use XML::Sablotron ();
use CGI ();

my $q;

sub handler {
  $q = new CGI(shift());
  my %credentials = hash_credentials();

  if (! $credentials{"l"} || ! defined($credentials{"l"})) {
    trace("Username credential is not defined. Aborting.", "error");
    exit(1);
  }
  
  my ($server, $server_ok, $err, $errstr);
  my $parser = new XML::DOM::Parser;
  my $version = VMware::Control::version();
  my $processor = new XML::Sablotron();
  my $want_version = "1.00";
  my $xsl = $ENV{ "style_DEFAULT" };
  my $xml;

  $server = &VMware::Control::Server::new(undef, 0, $credentials{"l"}, $credentials{"m"});
  ($server_ok, $err, $errstr) = connect_ctrl_obj($server);

  my $ret;
  my $xml;
  my $dir = $q->param("cwd");
  $dir ||= "/";

  if ($server_ok) {
    
    $ret = $server->exec("<exec><op>FileManager_ListDirTree</op><in><directory>$dir</directory></in></exec>");

    if (defined($ret)) {
      $ret = VMware::DOMAccess->newXML($ret);
      $ret->setValue("exec.out.selected", $dir);
      $xml = $ret->prettyPrint();

      $ret->dispose();
    }
  } else {
    # [ ! ] XXX Trace an error about the inability to connect to vmauthd 
    trace("$err: $errstr", "error");
  }

  if ($server_ok) {
    $server->disconnect( );
    trace( "server->disconnect() called ($server).", "notice");
  }

  trace("Xml: $xml", "debug"); 
  $processor->SetBase("file://" . $ENV{ "vmware_BASEHREF" });
  $processor->RunProcessor( "arg:/template", "arg:/data", "arg:/result",
                            undef,
                            [ "template", 
                            file_to_string($xsl), 
                            "data", 
                            $xml ] );
  return ( $processor->GetResultArg("result"),
           "Pragma"        => "no-cache",
           "Cache-Control" => "no-cache, must-revalidate",
           "Content-Type"  => "text/html; charset=iso-8859-1" );

  return ( $xml, 
           "Pragma"        => "no-cache",
           "Cache-Control" => "no-cache, must-revalidate",
           "Content-Type"  => "text/html; charset=iso-8859-1" );
}

1;
