# ---------------------------------------------------------------------------
# Copyright 2003 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SxCreateVm;
# File: VMware/Management/SxCreateVm.pm

use strict;

use VMware::Management::Util ();
use VMware::VmdbPerl ();
use VMware::Management::VmdbUtil ();
use VMware::Management::Session ();
use VMware::ExtHelpers qw(&internal_dirname &internal_basename);
use VMware::HConfig::CommonConstant qw(@VMX_CFG_EXTENSIONS);
use VMware::Management::SS::VMUtil ();

use CGI ();
my $q;

# ---------------------------------------------------------------------------

sub handler {
   $q = new CGI(shift());
   my $ctx = &VMware::Management::Session::get_temp_context();
   $ctx->SetCurrentPath("/");

   if ($q->param("op") =~ /^set$/i) {
      return set($ctx);
   }

   return data($ctx);
}


# ---------------------------------------------------------------------------

sub data {
   my $ctx = shift();
   my $data = "";

   # XXX Would be nice to get the per-user default here
   my $home = $ctx->GetString("/host2/#_mui/settingsState/val/defaultVMPath");
   $home = jsStringLiteral($home);

   $data .= qq{var home=$home;$/};

   # Server memory info -- duplicated from VmConfiguration::vmMemoryData
   my $total = $ctx->GetInt("/host2/#_mui/info/memory/sizeMb");
   my $avail = $ctx->GetInt("/host2/#_mui/info/memory/sizeAvailMb/");
   my @mxnew = ( 0 );
   foreach my $p (1 .. 2) {
      my $kbmax = $ctx->GetInt("/vmkernel/host/stats/memory/vmMaxSize/${p}p");
      push @mxnew, int($kbmax / 1024);
   }
   my $mxnew = "[" . join(",", @mxnew) . "]";
   $data .= qq{var memInfo={total:$total,avail:$avail,mxnew:$mxnew};$/};

   return html($data);
}


# ---------------------------------------------------------------------------

sub set {
   my $ctx = shift();
   my %credentials = VMware::Management::Util::hash_credentials();
   my $sep = ($^O eq "MSWin32" ? "\\" : "/");

   my $guestOS          = $q->param("guestOS") || "win2000Serv";
   my $location         = $q->param("location");
   my $name             = $q->param("name") || "New Virtual Machine";
   my $suspendDirectory = $q->param("suspendDirectory") || ".";
   my $num              = $q->param("num") || "";
   my $sizeMb           = $q->param("sizeMb") || 256;
   my $scsiCtlr         = $q->param("scsiCtlr") || "buslogic";

   # Append the config file name to the directory name
   $location .= ($location =~ /\Q$sep\E$/ ? "" : $sep) . "${guestOS}.vmx";

   my ($trnCtx, $reqPath) = cmdHead($ctx);

   # Location
   $trnCtx->SetString("${reqPath}in/file", "$location");
   $trnCtx->SetString("${reqPath}in/user", $credentials{"l"});
   $trnCtx->SetBool("${reqPath}in/createDir", 1);
   $trnCtx->SetBool("${reqPath}in/flags", 1);
   $trnCtx->SetBool("${reqPath}in/access/private", 1);

   # Guest Operating System
   $trnCtx->SetString("${reqPath}in/cfgVal/guestOS", $guestOS);

   # Display Name
   $trnCtx->SetString("${reqPath}in/cfgVal/name", $name);

   # Suspend Name
   $trnCtx->SetString("${reqPath}in/cfgVal/opt/checkpointCptConfigName",
                      suspendName($location));

   # Suspend Volume
   $trnCtx->SetString("${reqPath}in/cfgVal/opt/suspendDirectory",
                      $suspendDirectory);

   # Processors
   my $devPath = $trnCtx->GetAbsPath($trnCtx->NewArrayIndex("${reqPath}in/cfgVal/dev/#"));

   $trnCtx->SetString("${devPath}present", 1);
   $trnCtx->SetString("${devPath}class", "vcpu");
   $trnCtx->SetInt("${devPath}class/vcpu/num", $num);

   # Memory
   $trnCtx->SetString("${reqPath}in/cfgVal/dev/#_memory/key", "memory");
   $trnCtx->SetString("${reqPath}in/cfgVal/dev/#_memory/class", "memory");
   $trnCtx->SetInt("${reqPath}in/cfgVal/dev/#_memory/class/memory/sizeMb",
                   $sizeMb);

   # Floppy
   $trnCtx->SetString("${reqPath}in/cfgVal/dev/#_floppy0/class", "floppy");
   $trnCtx->SetInt("${reqPath}in/cfgVal/dev/#_floppy0/class/floppy/ix", 0);
   $trnCtx->SetString("${reqPath}in/cfgVal/dev/#_floppy0/media", "removable");
   $trnCtx->SetBool("${reqPath}in/cfgVal/dev/#_floppy0/media/removable/startConnected", 1);
   $trnCtx->SetString("${reqPath}in/cfgVal/dev/#_floppy0/class/floppy/hostif", "device");
   $trnCtx->SetString("${reqPath}in/cfgVal/dev/#_floppy0/class/floppy/hostif/device/hostdev", 
                      $^O eq "MSWin32" ? "A:" : "/dev/fd0");

   # CD-ROM
   $devPath = $trnCtx->GetAbsPath($trnCtx->NewArrayIndex("${reqPath}in/cfgVal/dev/#"));

   $trnCtx->SetString("${devPath}present", 1);
   $trnCtx->SetString("${devPath}class", "ideDev");
   $trnCtx->SetString("${devPath}class/ideDev/subclass", "cdrom");
   $trnCtx->SetInt("${devPath}class/ideDev/ctlrIx", 1);
   $trnCtx->SetInt("${devPath}class/ideDev/ix", 0);
   $trnCtx->SetString("${devPath}media", "removable");
   $trnCtx->SetBool("${devPath}media/removable/startConnected", 1);
   $trnCtx->SetString("${devPath}class/ideDev/subclass/cdrom/hostif", "device");
   $trnCtx->SetString("${devPath}class/ideDev/subclass/cdrom/hostif/device/hostdev",
                      $^O eq "MSWin32" ? "auto detect" : "/dev/cdrom");
   $trnCtx->SetBool("${devPath}class/ideDev/subclass/cdrom/hostif/device/exclusive", 0);
   $trnCtx->SetBool("${devPath}class/ideDev/subclass/cdrom/hostif/device/raw",
                    $^O eq "MSWin32" ? 1 : 0);

   # Network Adapter
   my ($hostif, $name) = nicDev($trnCtx);

   if (defined($hostif)) {
      $devPath = $trnCtx->GetAbsPath($trnCtx->NewArrayIndex("${reqPath}in/cfgVal/dev/#"));

      $trnCtx->SetString("${devPath}present", 1);
      $trnCtx->SetString("${devPath}class", "nic");
      $trnCtx->SetInt("${devPath}class/nic/ix", 0);

      if (defined($name)) {
         $trnCtx->SetString("${devPath}class/nic/devName", $name);

         # Use the host independent network label for $nic, if it's available.
         my $tmpCtx = &VMware::Management::Session::get_temp_context();
         my $path = "/host2/#_mui/config/network/#";
         while (defined($path = $tmpCtx->GetNextSibling($path))) {
            my $dev = $tmpCtx->GetString("${path}device");
            my $net = $tmpCtx->GetString("${path}network");
            
            if ($dev eq $name && defined($net) && $net ne "") {
               $trnCtx->SetString("${devPath}class/nic/networkName", $net);
               last;
            }
         }
      }

      $trnCtx->SetString("${devPath}class/nic/emulation", "vlance");
      $trnCtx->SetString("${devPath}class/nic/hostif", $hostif);
   }

   # SCSI Controllers
   $devPath = $trnCtx->GetAbsPath($trnCtx->NewArrayIndex("${reqPath}in/cfgVal/dev/#"));

   $trnCtx->SetBool("${devPath}present", 1);
   $trnCtx->SetString("${devPath}class", "scsiCtlr");
   $trnCtx->SetInt("${devPath}class/scsiCtlr/ix", 0);
   $trnCtx->SetString("${devPath}class/scsiCtlr/virtualDev", $scsiCtlr);

   # Display
   $devPath = $trnCtx->GetAbsPath($trnCtx->NewArrayIndex("${reqPath}in/cfgVal/dev/#"));

   $trnCtx->SetBool("${devPath}present", 1);
   $trnCtx->SetString("${devPath}class", "video");
   $trnCtx->SetInt("${devPath}class/video/depth", 16);

   my ($ok, $data) = cmdTail($trnCtx, $reqPath);

   return html($data);
}

# ---------------------------------------------------------------------------

sub cmdHead {
   my $ctx = shift();

   # Create a new context for this transaction to avoid
   # VMDB_E_DISTRIBUTED_COMMIT errors.
   my $trnCtx = &VMware::Management::Session::get_temp_context();
   $trnCtx->SetCurrentPath($ctx->GetCurrentPath());

   $trnCtx->BeginTransaction();

   # Initialize VMDB request.
   my $reqPath = $trnCtx->NewArrayIndex("/host2/#_mui/util/vm/cmd/##");
   $reqPath = $trnCtx->GetAbsPath($reqPath);
   $trnCtx->SetString("${reqPath}op", "createVM");
   $trnCtx->SetString("${reqPath}user",
                      &VMware::Management::Session::get_usertoken());

   return ($trnCtx, "${reqPath}op/createVM/");
}


# ---------------------------------------------------------------------------

sub cmdTail {
   my $trnCtx = shift();
   my $reqPath = shift();
   my $data = "";

   my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
                                                             $reqPath);

   if (! $ok) {
      $data .= error("Add Virtual Machine: $msg.");
   } else {
      my $cfgHash = $trnCtx->GetString("${reqPath}out/cfgHash");
      $data .= qq{var hash="$cfgHash";};
   }

   $reqPath =~ s{^(.*/cmd/\#\#.*?/).*}{$1};
   $trnCtx->LocalArrayUnset($reqPath, 1);

   return wantarray ? ($ok, $data) : $data;
}


# ---------------------------------------------------------------------------
#
# Return <file name>-<uniquifying hash> using djb2 hash algorithm.
#
# ---------------------------------------------------------------------------

sub suspendName {
   # Prevent Perl from converting integer to float in this routine.
   use integer;

   my $fqfn = shift(); # Fully qualified file name

   # $baseName is $fqfn stripped of its extension.
   my $baseName = internal_basename($fqfn);
   for (@VMX_CFG_EXTENSIONS) {
      if ($baseName =~ /\.$_$/) {
         $baseName =~ s/\.$_$//;
         last;
      }
   }

   my $hash = 5381;

   # Use the $fqfn path as the basis for the uniquifying hash.
   my @digits = map {ord($_)} split(/ */, internal_dirname($fqfn));

   # Include some randomness to avoid collisions.
   unshift(@digits, $$);
   unshift(@digits, time());
   unshift(@digits, int(128 * rand()));

   for (@digits) {
      $hash = $hash * 33 + $_;
   }

   return sprintf("%s-%08x", $baseName, $hash);
}


# ---------------------------------------------------------------------------

sub nicDev {
   my $ctx = shift;
   my $nic;

   # XXX Hard-code product ID if it's not available in VMDB (PR8862);
   my $prdId = $ctx->GetString("/host2/#_mui/productinfo/id") or "gsx";

   if ($prdId == /gsx/i) {
      # For GSX, always create a bridged network by default
      return ("bridged", undef);
   }

   # VMNICs
   my $subTree = VMware::Management::SS::VMUtil::GetVmkNetAdapters();
   my $vmnicStr = "";
   for ($subTree->listElementNames("netAdapters")) {
      if ($subTree->getValue("netAdapters.$_.nicteam")) {
         next;
      }
      $nic = $subTree->getValue("netAdapters.$_.name");
      if (defined($nic) && $nic ne "") {
         VMware::Management::Util::trace("NIC: $nic", "debug");
         last;
      }
   }
   $subTree->dispose();

   if (defined($nic) && $nic ne "") {
      return ("monitor_dev", $nic);
   }

   # Bonds
   $subTree = VMware::Management::SS::VMUtil::GetVmkNetBondAdapters();
   my $bondStr = "";
   for ($subTree->listElementNames("netAdapters")) {
      if (scalar($subTree->listElementNames("netAdapters.$_.slaves"))) {
         $nic = $subTree->getValue("netAdapters.$_.name");
         if (defined($nic) && $nic ne "") {
            VMware::Management::Util::trace("NIC: $nic", "debug");
            last;
         }
      }
   }
   $subTree->dispose();

   return defined($nic) && $nic ne "" ? ("monitor_dev", $nic) : (undef, undef);
}


# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "error", undef, $pkg, $fn, $ln);
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "warning", undef, $pkg, $fn, $ln);
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $str is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $str = shift();
   $str =~ s/\\/\\\\/g;
   $str =~ s/("|')/\\$1/g;
   my $lf = $/;
   $str =~ s/$lf/\\n/g;

   return qq{"$str"};
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=[];var wrn=[];$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}

1;
