# ---------------------------------------------------------------------------
# Copyright 2003 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SxLogViewer;
# File: VMware/Management/SxLogViewer.pm

use strict;

use VMware::ExtHelpers qw( &Warning &clearWarnings );
use VMware::Management::Util qw( &hash_credentials );
use VMware::Management::SS::VMUtil;

use CGI();
my $q;

my $logDir = "/var/log";

sub handler {
   clearWarnings();

   $q = new CGI(shift());
   my %credentials = hash_credentials();

   my $user = $credentials{l};
   my $password = $credentials{m};

   if (! VMware::Management::Session::user_is_admin()) {
      return html(error("Log Viewer: Permission denied."),
                  "<p>Log Viewer: Permission Denied</p>");
   }

   return data();
}



# ---------------------------------------------------------------------------

sub data {
   my $log = $q->param("log");
   my $data = "";
   my $html = "";

   if ($log ne "") {
      my $filename = "$logDir/$log";
      my ($result, $contents) = VMware::Management::SS::VMUtil::ReadLastN($filename, 500);

      VMware::Management::Util::trace("$contents", "debug");

      if ($result == 1) {
         for (split("$/", $contents)) {
            if (m/ALERT|PanicWork|Oops:/) {
               $_ = qq{<b><FONT COLOR="#FF0000">$_$/</FONT></b>};
            }

            $html .= qq{<div>$_</div>$/};
         }
      } else {
         $html = qq{<div class="cfg">} .
            qq{<table border="0" cellpadding="0" cellspacing="0"><tr>} .
            qq{<td class="key"><b>} .
            qq{$filename: $contents.} .
            qq{</b></td>} .
            qq{</tr></table></div>};
      }

      if ($html eq "") {
         $html = qq{<div class="cfg">} .
         qq{<table border="0" cellpadding="0" cellspacing="0"><tr>} .
         qq{<td class="key"><b>} .
         qq{$filename: No entries.} .
         qq{</b></td>} .
         qq{</tr></table></div>};
      }
   }

   return html($data, $html);
}


# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "error", undef, $pkg, $fn, $ln);
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "warning", undef, $pkg, $fn, $ln);
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $str is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $str = shift();
   $str =~ s/\\/\\\\/g;
   $str =~ s/("|')/\\$1/g;
   my $lf = $/;
   $str =~ s/$lf/\\n/g;

   return qq{"$str"};
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $html = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   $html = qq[<html><head><title></title><script>$/] .
      qq[var err=[];var wrn=[];$/] .
      qq[$data$/</script>$/] .
      qq[<script src="/vmware/src/init.js"></script>$/] .
      qq[<script src="/vmware/src/xuaLib.js"></script>$/] .
      qq[<link rel="stylesheet" href="/vmware/css/page.css" type="text/css" />$/] .
      qq[</head><body id="bodyObj" onload="$cb">$/$html</body></html>];



   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}


1;
