# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SxLogin;
# File: VMware/Management/SxLogin.pm

use strict;

use VMware::Control '1.00';
use VMware::Control::Server ();
use VMware::Control::Profiler;
use VMware::HConfig::Constant qw( $URL_INDEX
                                  $URL_OVERVIEW
                                  $URL_LOGOUT );
use VMware::HConfig::SSHostRemote();
use VMware::Management::Util qw( trace
                                 connect_ctrl_obj
                                 hash_credentials
                                 string_to_file
				 jsStringLiteral );

use CGI ();
use MIME::Base64 ();
use VMware::Management::Session;

my $q;
my $p = VMware::Control::Profiler::getInstance("mui");
srand(time() ^ ($$ + ($$ << 15)));


# ---------------------------------------------------------------------------
#  handler
# ---------------------------------------------------------------------------
sub handler {
   my $stub = $p->profile();
   my ($auth_ok, $user, $err, $str, $next, $product);
   my $k_cookie = "";
   my $s_cookie = "";

   $q = new CGI(shift());

   # We have a username. Try to authenticate the given credentials.
   if ($q->param("l")) {
      chomp($user = $q->param("l"));
      if ($user ne "") {
        ($auth_ok, $err, $str, $product) = authenticate();
      }
   }

   # The credentials are OK. Figure out where to go next.
   if (defined($auth_ok) && $auth_ok) {
      my $username = MIME::Base64::decode_base64($q->param("l"));
      my $password = MIME::Base64::decode_base64($q->param("m"));
      my $isESX = (($product eq "ESX") or 0);
      my $host = undef;

      if ($isESX) {
         $host = VMware::HConfig::SSHostRemote->new(undef, 0, $username, $password);
      }

      # Something is extraordinarily wrong.
      if ($isESX && !defined($host) && $ENV{HTTP_REFERER} !~ /\?ctx=wizard$/i) {
         $str = "Internal Error: Could not create remote host object.";
      } elsif ($isESX && !$host->isVMKernel() && $ENV{HTTP_REFERER} !~ /\?ctx=wizard$/i) {
         # There's no vmkernel and the user is not root.

         # XXX: Hack
         $ENV{HTTP_COOKIE} = VMware::Management::Session::validate_session(
	    undef,
	    $q->param("l"),
	    $q->param("m")
	 );

         if (! VMware::Management::Session::user_is_admin()) {
            $str = qq(Your system configuration is not complete, and you ).
                   qq(will not be able to run virtual machines until the ) .
                   qq(VMware ESX Server kernel is loaded. Please log in as ) .
                   qq(a user with administration privileges to complete ) .
                   qq(your system configuration.);

            # XXX: Hack
            my %cookies = VMware::Management::Util::hash_cookies();
            VMware::Management::Session::delete_session($cookies{"vmware.mui.sid"});
            $ENV{HTTP_COOKIE} = "";
         } else {
            # Go to the system configuration pages next.
            ($k_cookie, $s_cookie) = initialize_session();
            $next = "/vmware/en/esxHostConfigWizard.html"; # $URL_INDEX;
         }
      } else {
         # We're all set. Set our course and initalize the cookies.
         my $loc = "/monitor/index.pl";
         ($k_cookie, $s_cookie) = initialize_session();

         return ("",
                 "Location" => $loc,
                 "kid-Cookie" => $k_cookie,
                 "sid-Cookie" => $s_cookie);

      }
   }

   if ($next) {
      # The client should go to an alternate page (i.e., /vmware/config).
      return (html($user, $err, $str, $next),
              "Pragma"        => "no-cache",
              "Cache-Control" => "no-cache",
              "kid-Cookie" => $k_cookie,
              "sid-Cookie" => $s_cookie,
              "Content-Type"  => "text/html; charset=iso-8859-1");
   } else {
      # Login failed or no attempt to login has been made.
      return (html($user, $err, $str, $next),
              "Pragma"        => "no-cache",
              "Cache-Control" => "no-cache",
              "Content-Type"  => "text/html; charset=iso-8859-1");
   }
}


# ---------------------------------------------------------------------------
#  authenticate
# ---------------------------------------------------------------------------
sub authenticate {
   my $stub = $p->profile();
   my $username = MIME::Base64::decode_base64($q->param("l"));
   my $password = MIME::Base64::decode_base64($q->param("m"));
   my $product = undef;

   my $server = VMware::Control::Server::new(undef, 0, $username, $password);
   my ($server_ok, $err, $str) = connect_ctrl_obj($server);

   if ($server_ok) {
      $product = VMware::Management::Util::getProductName($server);
   }

   if (defined($server)) {
      $server->disconnect();
   }

   return ($server_ok, $err, $str, $product);
}


# ---------------------------------------------------------------------------
#  initialize_session
# ---------------------------------------------------------------------------
sub initialize_session {
   my $stub = $p->profile();
   my $kc = "vmware.mui.kid=";
   my $sc = undef;

   my $username = MIME::Base64::decode_base64($q->param("l"));
   $sc = VMware::Management::Session::validate_session(undef,
						       $q->param("l"),
                                                       $q->param("m"));

   if (VMware::Management::Util::use_vmdb()) {
      return (undef, $sc);
   }
   
   # Munge username:password:sid.
   $kc .= MIME::Base64::encode_base64($q->param("l") . ":" .
                                      $q->param("m") . ":$sc", "");
   chomp($kc);

   return ("$kc;path=/;", "$sc");
}


# ---------------------------------------------------------------------------
#  html
# ---------------------------------------------------------------------------
sub html {
   my ($user, $err, $str, $next) = @_;

   if (defined($err)) {
      trace("$err: $str", "error");
   } elsif ($ENV{"vmware_SESSION_LENGTH"} == 0) {
      $str = "The VMware Management Interface is closed. No logins permitted.";
   } elsif (! defined($str))  {
      my %cookies = &VMware::Management::Util::hash_cookies();
      if (defined($cookies{"vmware.mui.sid"})) {
	 # There's a session, but it has expired.
         $str = "Your session has expired. Please login.";
      }
   }

   $user = defined($user) ? jsStringLiteral($user) : "null";
   $err  = defined($err)  ? jsStringLiteral($err)  : "null";
   $str  = defined($str)  ? jsStringLiteral($str)  : "null";
   $next = defined($next) ? jsStringLiteral($next) : "null";

   my $head = qq{<html><head><title>Login: VMware Management Interface</title><script>$/};
   my $body = qq{var user=$user;var err=$err;var str=$str;var next=$next;$/};
   my $tail = qq[</script></head><body bgcolor="#336699" onload="try{if(parent.loginCb)parent.loginCb(self);}catch(e){;}"></body></html>$/];

   if ($ENV{"vmware_DEBUG_LEVEL"} > 2) {
      my $file = &VMware::Management::Util::tmpdir() . "monitor.html";
      string_to_file($head . $body . $tail, $file, 1);
   }

   return "$head$body$tail";
}

1;
