# ---------------------------------------------------------------------------
# Copyright 2003 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SxPing;
# File: VMware/Management/SxPing.pm

use VMware::HConfig::SSHostRemote();
use VMware::Management::Util ();

use CGI ();
my $q;

sub handler {
   my $r = shift();
   my @args = $r->method() eq "GET" ? $r->args() : $r->content();
   my %params = ();

   while(my ($name, $value) = splice(@args, 0, 2)) {
      $params{$name} .= ($params{$name} ? "," : "") . $value;
   }

   if ($r->method() eq "POST") {
      $r->method("GET");
      $r->method_number(M_GET);
      $r->headers_in->unset("Content-length");
   }

   $r->no_cache(1);
   $r->content_type("text/html; charset=utf-8");
   $r->send_http_header();

   $q = new CGI(\%params);

   my $host = VMware::HConfig::SSHostRemote->new();

   if (! defined($host)) {
      $r->print(html(error("No host.")));
   } else {
      my $vmkernel = $host->isVMKernel() ? "true" : "false";
      $r->print(html(qq{data.vmk=$vmkernel;}));
   }

   return OK;
}

# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "error", undef, $pkg, $fn, $ln);
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "warning", undef, $pkg, $fn, $ln);
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $str is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $str = shift();
   $str =~ s/\\/\\\\/g;
   $str =~ s/("|')/\\$1/g;
   my $lf = $/;
   $str =~ s/$lf/\\n/g;

   return qq{"$str"};
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.pingCb) parent.pingCb(self);";

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=[];var wrn=[];var data={};$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return $html;
}

1;
