# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SxShutdown;
# File: VMware/Management/SxShutdown.pm

use strict;

use VMware::Management::Util ();

use VMware::HConfig::SSHostRemote;
use VMware::ExtHelpers ();

use VMware::Management::SS::ESXConfig;

use CGI();

sub handler {
   VMware::ExtHelpers::clearWarnings();

   my $q = new CGI(shift());
   my $req = $q->param("req");
   my $msg = $q->param("msg");
   my $data = "";

   VMware::Management::Util::trace($req, "debug");
   VMware::Management::Util::trace($msg, "debug");

   my %credentials = VMware::Management::Util::hash_credentials();
   my $user = $credentials{l};
   my $pass = $credentials{m};

   if (! VMware::Management::Session::user_is_admin()) {
      my $act = ($req =~ /^reboot$/i) ? "Restart" : "Shut Down";
      $data .= error("$act: Permission denied.");
      return html($data);
   }

   my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $pass);

   if (! defined($host)) {
      my $act = ($req =~ /^reboot$/i) ? "Restart" : "Shut Down";
      $data .= error("$act: Could not create remote host object.");
      return html($data);
   }

   if ($req eq "reboot") {
      $host->rebootSystem($msg);
      return html($data);
   } elsif ($req eq "halt") {
      $host->haltSystem($msg);
      return html($data);
   }

   return html(error("Restart/Shut Down: Unknown request: " . $req));
}

# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "error", undef, $pkg, $fn, $ln);
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "warning", undef, $pkg, $fn, $ln);
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $str is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $str = shift();
   $str =~ s/\\/\\\\/g;
   $str =~ s/("|')/\\$1/g;
   my $lf = $/;
   $str =~ s/$lf/\\n/g;

   return qq{"$str"};
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   my @warnings = VMware::ExtHelpers::getWarnings();
   for (@warnings) {
      $data .= warning($_);
   }
   VMware::ExtHelpers::clearWarnings();

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=[];var wrn=[];$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}

1;
