# ---------------------------------------------------------------------------
# Copyright 2003 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SxVmStartupShutdown;
# File: VMware/Management/SxVmStartupShutdown.pm

use strict;

use VMware::Management::Util ();
use VMware::Management::VmdbUtil ();

use CGI ();

my $q;

sub handler {
   $q = new CGI(shift());
   my $ctx = &VMware::Management::Session::get_temp_context();
   my $data = "";

#  for ($q->param()) {
#      VMware::Management::Util::trace("OPT $_: " . $q->param($_),"debug");
#   }

   $ctx->SetCurrentPath("/");

   return data($ctx, $data);
}



# ---------------------------------------------------------------------------

sub data {
   my $ctx = shift();
   my $data = shift();
   my $msg = "";

   if ($q->param("hdnVmStartupOp") eq "editorRetrieve") {
     $data .= retrieveVmStartupSequence($ctx);
   } elsif ($q->param("hdnVmStartupOp") eq "editorSet") {
     $data .= processVmStartupSequence($ctx);
   } elsif ($q->param("hdnVmStartupOp") eq "defaultIntervals") {
     $data .= processVmStartupDefaults($ctx);
   }

   return html($data);
}


# ---------------------------------------------------------------------------

# pass js var declarations and assignements in as data
# pass onload stuff as the second parameter cb

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=[];var wrn=[];$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}

# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "error", undef, $pkg, $fn, $ln);
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "warning", undef, $pkg, $fn, $ln);
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}

# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $str is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $str = shift();
   $str =~ s/\\/\\\\/g;
   $str =~ s/("|')/\\$1/g;
   my $lf = $/;
   $str =~ s/$lf/\\n/g;

   return qq{"$str"};
}



# ---------------------------------------------------------------------------

sub cmdHead {
   my $ctx = shift();
   my $strOp = shift();

   # Create a new context for this transaction to avoid
   # VMDB_E_DISTRIBUTED_COMMIT errors.
   my $trnCtx = &VMware::Management::Session::get_temp_context();
   $trnCtx->SetCurrentPath($ctx->GetCurrentPath());

   $trnCtx->BeginTransaction();


   # Initialize VMDB request.
   my $cmdIdxPath = $trnCtx->NewArrayIndex("/host2/#_mui/cmd/##");
   my $reqPath = $trnCtx->GetAbsPath($cmdIdxPath);
   $trnCtx->SetString("${reqPath}op", $strOp);
   $trnCtx->SetString("${reqPath}user",
                      &VMware::Management::Session::get_usertoken());

   return ($trnCtx, $reqPath, "${reqPath}op/" . $strOp ."/" );

}

# ---------------------------------------------------------------------------

sub cmdTail {
   my $trnCtx = shift();
   my $reqPath = shift();
   my $data = "";

   my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
                                                             $reqPath);

   if (! $ok) {
     $data .= error("Error setting system defaults for VMs startup : $msg.");
   }

   return wantarray ? ($ok, $data) : $data;
}

# ---------------------------------------------------------------------------
# Note ! The path to unset for commands ends at command localArrayIndex /cmd/##x/

sub cmdUnset {
  my $trnCtx = shift();
  my $cmdIdxPath = shift();
  $trnCtx->LocalArrayUnset($cmdIdxPath, 1);
}

# ---------------------------------------------------------------------------

sub processVmStartupDefaults {

  my $ctx = shift();
  my ($ok, $trnCtx, $cmdIdxPath, $reqPath, $data);

#  for ($q->param()) {
#    VMware::Management::Util::trace("OPT $_: " . $q->param($_),"debug");
#  }

  # deal with checkboxes first

  my $autoStartEnabled = ($q->param("autostartAll") eq "") ? 0 : 1;

  ($trnCtx, $cmdIdxPath, $reqPath) = cmdHead($ctx,"setHostSettings");

  $trnCtx->SetBool("${reqPath}in/val/autoStartEnabled", $autoStartEnabled);

  # XXX In case the enable autostart checkbox is unchecked, the rest of the client side
  # controls are disabled, which in turn means no parameters are sent. If future MUI
  # implementations might unrelates them remove the following if statement


  if ($autoStartEnabled != 0) {
    if ( $q->param("autostartDelayAll") ne "" ) {
      $trnCtx->SetInt("${reqPath}in/val/autoStartDefaultStartDelay",
		      $q->param("autostartDelayAll"));
    }

    if ( $q->param("autostopDelayAll") ne "" ) {
      $trnCtx->SetInt("${reqPath}in/val/autoStartDefaultStopDelay",
		      $q->param("autostopDelayAll"));
    }

    my $autoStartContTools = ($q->param("autostartContinueWhenToolsRun") eq "") ? 0 : 1;

    $trnCtx->SetBool("${reqPath}in/val/autoStartDefaultContinueWhenToolsRun",
		     $autoStartContTools);

  }

  ($ok, $data) = cmdTail($trnCtx, $reqPath);
  cmdUnset($trnCtx, $cmdIdxPath);

  return $data;

}



# ---------------------------------------------------------------------------

sub retrieveVmStartupSequence {

   my $ctx = shift();
   my ($ok, $trnCtx, $cmdIdxPath, $reqPath, $data);
   my ($genPath,$genNumber);
   my ($orderedPath, $unorderedPath, $disabledPath);

  ($trnCtx, $cmdIdxPath, $reqPath) = cmdHead($ctx,"getAutoStartInfo");

   cmdTail($trnCtx, $reqPath);

   $genPath = $reqPath."out/generationNo/";
   $orderedPath = $reqPath."out/autoStartEnabled/startOrder/#";
   $unorderedPath = $reqPath."out/autoStartEnabled/unOrdered/#";
   $disabledPath = $reqPath."out/autoStartDisabled/#";

   # generation number

   $genNumber = $trnCtx->GetInt($genPath);
   $data .= qq{genNum = "$genNumber";};

   # VMs arrays
   $data .= createVmJsArray($trnCtx,$orderedPath,"arrOrdered");
   $data .= createVmJsArray($trnCtx,$unorderedPath,"arrUnordered");
   $data .= createVmJsArray($trnCtx,$disabledPath,"arrDisabled");

   cmdUnset($trnCtx, $cmdIdxPath);

   return $data;

}

# ---------------------------------------------------------------------------

sub processVmStartupSequence {

   my $ctx = shift();
   my ($ok, $trnCtx, $cmdIdxPath, $reqPath, $data);


  ($trnCtx, $cmdIdxPath, $reqPath) = cmdHead($ctx,"setAutoStartInfo");

   my $genNum = $q-> param("hdnGenNum");
   $trnCtx->SetInt("${reqPath}in/generationNo/", $genNum);

   setVmList($trnCtx,$reqPath,"hdnOrdVms");
   setVmList($trnCtx,$reqPath,"hdnUnordVms");
   setVmList($trnCtx,$reqPath,"hdnDsblVms");


  ($ok, $data) = cmdTail($trnCtx, $reqPath);
   cmdUnset($trnCtx, $cmdIdxPath);

   return $data;
}

# ---------------------------------------------------------------------------

sub setVmList {

  my $ctx = shift();
  my $reqPath = shift();
  my $listName = shift();

  my $autoStartPostfix = "/autostart/";
  my $autoStopPostfix = "/autostop/";
  my $cfgFilePostfix = "/vmCfgFile/";
  my $vmsPath;

  my $vmList = $q->param($listName);

  if ($listName eq "hdnOrdVms") {
    $vmsPath = "in/startOrder/#";
  } elsif ($listName eq "hdnUnordVms") {
    $vmsPath = "in/unOrdered/#";
  } elsif ($listName eq "hdnDsblVms") {
    $vmsPath = "in/disabledVms/#";
  }

  my %vms = VMware::Management::Util::hashQueryString($vmList);
  my $vmListSize = keys %vms;

  if ($vmListSize > 0) { # Corresponding VMs were sent by the client

    my @tmpKeys = keys %vms;
    my @tmpValues = values %vms;
    my ($tmpKey, $tmpVal);

    my $vmdbPath;
    my ($cfgFile, $autostart, $autostop);

    my $tmpCtx = &VMware::Management::Session::get_temp_context();

    while(@tmpKeys){

      $tmpKey = pop(@tmpKeys);
      $tmpVal = pop(@tmpValues);
      ($vmdbPath, $autostart) = split(",",$tmpVal);

      $autostop = $tmpCtx->GetString($vmdbPath."vmx/cfgState/val/autostop/");
      $cfgFile  = $tmpCtx->GetString($vmdbPath."cfgPath/");

      if($autostart eq "true") {
	$autostart = "poweron";
	if($autostop eq "none") {
	  $autostop = "softpoweroff";
	}
      } else {
	$autostart = "none";
      }

      $ctx->SetString("${reqPath}${vmsPath}${tmpKey}${autoStartPostfix}", $autostart);
      $ctx->SetString("${reqPath}${vmsPath}${tmpKey}${autoStopPostfix}", $autostop);
      $ctx->SetString("${reqPath}${vmsPath}${tmpKey}${cfgFilePostfix}", $cfgFile);
      # XXX collect errors here
    }
  }
}

# ---------------------------------------------------------------------------

sub createVmJsArray {
  my $ctx = shift();
  my $vmPath = shift();
  my $arrName = shift();

  my $data = qq{$arrName = new Array;};
  my ($vmdbPath, $vmNamePath, $vmName, $vmCfgPath);

  while (defined($vmPath = $ctx->GetNextSibling($vmPath))) {
    $vmdbPath  = $ctx->GetString($vmPath);
    $vmNamePath = $vmdbPath . "vmx/cfgState/val/name/";
    $vmName = $ctx->GetString($vmNamePath);
    if($vmName eq "") {
      $vmCfgPath = $vmdbPath . "cfgPath/";
      $vmName = $ctx->GetString($vmCfgPath);
    }
    $data .= $arrName."[".$arrName.".length] = ";
    $data .= qq{{"id":"$vmdbPath","name":"$vmName"};};
  }

  return $data;
}

1;
