#!/usr/bin/perl
# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::VMnew;
# File: VMware/Management/VMnew.pm

# XXX TODO: A few things still remain to be fixed:
#   1. Changes in vmnew.js::SetOption to include '256 GB' as a disk size
#      option for scsi disks and keep it up to 128 GB for ide. 
#   2. The code in this file needs cleanup. The VM creation code should
#      move to an exec handler in serverd.

use strict;

use VMware::Control '1.00';
use VMware::Control::Server;
use VMware::Control::VM;
use VMware::Control::Profiler;
use VMware::Management::Util;
use VMware::DOMAccess;
use CGI;

use XML::DOM;
use XML::Sablotron;


my $debug = 1;
my $remote = 1;
#my $query;


my $doc;

my @warnings = ();

my $server = undef;

# We collect this information upfront during Initialize() and then
# don't have to go to the server all the time. 
my $hostInfo = undef; # XML tree representing the Host we are targetting.
my $hostOS = undef;   # OS on the remote Host that we are targetting.
my $fileSeparator = undef;
my $vmDefaultDir  = undef;
my $configFileExt = undef;
my $gVirtualHWVersion = undef;
my $gDiskExtension = ".vmdk";

my $gDefaultFloppyFilename;
my $gDefaultCDromFilename;

my $p = VMware::Control::Profiler::getInstance("mui");

sub handler {
  my $stub = $p->profile();

  my $query = new CGI;
  
  my $os = $query->param('os');

  my @ret;

  my %credentials = hash_credentials( );

  # Contact the server and get some information upfront
  $server = &VMware::Control::Server::new(undef, 0, 
                                          $credentials{ "l" }, 
                                          $credentials{ "m" } );
  if (!$server || !$server->connect()) {
      trace("Cannot connect", "debug");
      return ReturnError("Could not connect to server");
  }
  &Initialize($credentials{ "l" });

  # If $os is defined, then the user has filled out the new VM form and we
  # should try to create the requested VM. Otherwise we need to return a
  # default form. 
  if (defined($os)) {
      @ret = &NewVM;
  } else {
      if (!defined($vmDefaultDir)) {
	  trace("vmDefaultDir not defined\n" , "debug");
	  @ret = ReturnError("Could not get default VM directory");
      } else {
          my $doc = VMware::DOMAccess->new("input");

          $doc->setValue("input.defaults.common.cdrom.filename", $gDefaultCDromFilename);
          $doc->setValue("input.defaults.common.floppy.filename", $gDefaultFloppyFilename);
          
          $doc = &AppendServerInfo($doc);
          $doc = &AppendDefaults($doc);
          my $xml = $doc->prettyPrint();
          
          @ret = &emit_xml($xml);
      }
  }

  if (defined($hostInfo)) {
      $hostInfo->dispose();
  } 
  if ( $server && $server->is_connected() ) {
      $server->disconnect( );
      trace( "server->disconnect( ) called ($server).", "notice" );
  }

  return @ret;
}

sub AppendServerInfo
{
    my $doc = shift;
    my %credentials = hash_credentials( );
    my $owner = $credentials{ "l" };
    $doc->setValue("input.username", $owner);
    $doc->setValue("input.build", get_build());
    $doc->setValue("input.homeDir", $vmDefaultDir);
    $doc->setValue("input.hostOS", $hostOS);
    my ($host, $domain) = VMware::Management::Util::get_host_and_domain(); 
    $doc->setValue("input.host", $host);
    $doc->setAttribute("input.host", "domain", $domain);
    return $doc;
}

sub AppendDefaults
{
    my $stub = $p->profile();
    my $doc = shift();
    my $hostRAM = &GetHostRAM;

    my $req = VMware::DOMAccess->new("exec");
    $req->setValue("exec.op", "VMConfigDefaults_GetAll");
    my $resp = VMware::DOMAccess->newXML(request($req));
    
    if ($resp->getValue("exec.returncode")) {
        $doc->addSubTree("input.defaults", $resp->getSubTree("exec.out.guests"));
    } else {
	trace("AppendDefaults return code FALSE when getting defaults.\n" , "error");
    }

    $req->dispose();
    $resp->dispose();

    return $doc;
}

sub emit_xml {
    my $stub = $p->profile();
    my $xml = shift();
    my $xsl = $ENV{ "style_DEFAULT" };

    $xml = qq(<?xml version="1.0" encoding="utf-8"?>) . $xml;

    if ( $ENV{ "vmware_DEBUG_LEVEL" } > 2 ) {
	my $xmlfile = &VMware::Management::Util::tmpdir() . "vmnew.xml";
	string_to_file( $xml, $xmlfile, 1 );
    }

    my $processor = new XML::Sablotron();

    $processor->SetBase( "file://" . $ENV{ "vmware_BASEHREF" } );
    $p->start("XML::Sablotron::RunProcessor");
    $processor->RunProcessor( "arg:/template", "arg:/data", "arg:/result",
                            undef,
	      [ "template", file_to_string( $xsl ), "data", $xml ] );
    $p->end();
    my @ret = ( $processor->GetResultArg("result"),
                "Content-Type"  => "text/html; charset=utf-8",
                "Pragma"        => "no-cache",
                "Cache-Control" => "no-cache" );

    return @ret;
}

sub isAbsolutePath($) {
    my ($file) = @_;

    if ( ($hostOS =~ /MSWin32/i) && 
         (($file =~ /^[a-zA-Z]:\\/) || ($file =~ /^\\\\/)) ) {
      return 1;
    }

    if ( ($hostOS =~ /Linux/i) &&
         (substr($file, 0, 1) eq '/') ) {
        return 1;
    }

    return 0;
}
 
sub isPathTooLong($) {
    my $dir = shift;

    my $doc = VMware::DOMAccess->new("exec");
    $doc->setValue("exec.op", "VMPathTooLong");
    $doc->setValue("exec.in.config", $dir);
    my $ret = VMware::DOMAccess->newXML(request($doc));

    return ($ret->getValue("exec.returncode") eq "1");
}

sub validateInputParameters($$$$) {
    my $stub = $p->profile();
    my ($diskSize, $memSize, $displayname, $dir) = @_;

    if ($diskSize !~ /^[0-9]+$/) {
        return (0, "Invalid value of Disk Size: $diskSize ");
    }
    if ( ($memSize !~ /^[0-9]+$/) || ($memSize == 0) ) {
        return (0, "Invalid value of Memory Size: $memSize ");
    }
    if ($memSize % 4) {
        return (0, "Invalid value of Memory Size: $memSize. " .
                   "Memory size must be a multiple of 4.");
    }

    if (! isAbsolutePath($dir)) {
        return (0, "The virtual machine directory path is not absolute. You specified: \"$dir\"");
    }

    if (isPathTooLong($dir)) {
        return (0, "The virtual machine directory path is too long. You specified: \"$dir\"");
    }

    if ($displayname !~ /\S+/) {
        return (0, "You need to specify a display name for the virtual machine");
    }

    return 1;
}

sub NewVM {
    my $stub = $p->profile();
    my $ret;
    my $err;
    my $query = new CGI;
    my %credentials = hash_credentials( );
    my $diskSize = $query->param('disksize');
    my $displayname = $query->param('displayname');
    my $memSize = $query->param('memsize');
    my $os = $query->param('os');
    my $dir = $query->param('dir');
    my $scsi = $query->param('scsi');
    my $cfgname = $dir . $fileSeparator . $os;
   
    my ($ok, $err) = validateInputParameters($diskSize, $memSize, $displayname, $dir);
    if (! $ok) {
        return ReturnError($err);
    }
    &CheckDiskSize($dir, $diskSize);

    $ret = &CheckRAM($memSize);
    if(!OK($ret))
    {
        my $err = qq (The amount of RAM you have selected ($memSize MB) is
                      greater than what is available on your system.
                      );
        return ReturnError($err);
    }

    $ret = &MakeConfigDirectory($dir);
    if(!OK($ret))
    {
	my $err_doc = VMware::DOMAccess->newXML($ret);
	my $err = "Error making directory \"$dir\".  " .
          $err_doc->getValue("exec.messages.msg");
	$err_doc->dispose();
        return ReturnError($err);
    }

    my $cfg = &GenerateConfig;
    my $cfgFileName = $cfgname . $configFileExt;
    $ret = &WriteConfig($cfgFileName, $cfg);
    if(!OK($ret))
    {
	my $err_doc = VMware::DOMAccess->newXML($ret);
	my $err = "Error writing config file \"$cfgFileName\". " .
          $err_doc->getValue("exec.messages.msg");
	$err_doc->dispose();
        return ReturnError($err);
    }

    $ret = &CreateDisk($cfgname, $diskSize, $os, $scsi);
    if(!OK($ret))
    {
	my $err_doc = VMware::DOMAccess->newXML($ret);
	my $err = "Error writing disk files.  " .
          $err_doc->getValue("exec.messages.msg");
	$err_doc->dispose();
        return ReturnError($err);
    }

    $ret = &RegisterVM($cfgFileName);
    if (!OK($ret))
    {
	my $err_doc = VMware::DOMAccess->newXML($ret);
	my $err = $err_doc->getValue("exec.messages.msg");
        $err_doc->dispose();
        if ($err !~ /^Virtual machine already exists/) {
          return ReturnError($err);
        }
    }        

    return &ResponsePage($cfgFileName);
}

sub OK
{
    my $xml = shift();
    my $doc = VMware::DOMAccess->newXML($xml);
    return ($doc->getValue("exec.returncode") eq "1");
}

# XXX Hack to get better error reporting
sub PermissionDenied
{
    my $xml = shift();
    return( $xml =~ /[Pp]ermission [Dd]enied/ );
}

sub Initialize($) {
    my $stub = $p->profile();
    my ($username) = @_;

    my $doc = VMware::DOMAccess->new("exec");
    $doc->setValue("exec.op", "GSXHostInfo_Query");
    $doc->setValue("exec.in.username", $username);
    $hostInfo = VMware::DOMAccess->newXML(request($doc));
    $hostOS = $hostInfo->getValue("exec.out.hostOS");
    $vmDefaultDir = $hostInfo->getValue("exec.out.vmDefaultDir");
    $vmDefaultDir =~ s/\\/\\\\/g;

    if ($hostOS =~ /MSWin32/i) {
        $fileSeparator = "\\";
	$configFileExt = ".vmx";
        $gDefaultFloppyFilename = "A:";
        $gDefaultCDromFilename = "auto detect";
    } else {
        $fileSeparator = "/";
	$configFileExt = ".cfg";
        $gDefaultFloppyFilename = "/dev/fd0";
        $gDefaultCDromFilename = "/dev/cdrom";
    }

    $doc->dispose();
    $gVirtualHWVersion = undef;  # Will be initialized in GetConfigVersion()
    return;
}

sub RegisterVM($) {
    my $stub = $p->profile();
    my ($cfgFileName) = @_;

    my $doc = VMware::DOMAccess->new("exec");
    $doc->setValue("exec.op", "VMRegister");
    $doc->setValue("exec.in.config", $cfgFileName);
    my $resp = request($doc);

    $doc->dispose();
    return $resp;
}

sub GetConfigVersion {
    my $stub = $p->profile();
	       
    my $doc = VMware::DOMAccess->new("exec");
    $doc->setValue("exec.op", "VMConfigDefaults_GetConfigVersion");
    my $resp = VMware::DOMAccess->newXML(request($doc));
	       
    my $cvVar = $resp->getValue("exec.out.configVersion.variable");
    my $cvVal = $resp->getValue("exec.out.configVersion.value");
    my $hvVar = $resp->getValue("exec.out.virtualHWVersion.variable");
    my $hvVal = $resp->getValue("exec.out.virtualHWVersion.value");
    
    $gVirtualHWVersion = $hvVal;

    $doc->dispose();
    $resp->dispose();

    return ($cvVar, $cvVal, $hvVar, $hvVal);
}

sub CheckDiskSize 
{
    my $stub = $p->profile();
    my $dir = shift ();
    my $disksize = shift ();

    my $doc = VMware::DOMAccess->new("exec");
    $doc->setValue("exec.op", "GSXHostInfo_GetDiskSpace");
    $doc->setValue("exec.in.directory", $dir);

    my $resp = VMware::DOMAccess->newXML(request($doc));
    # Disk space returned is in MB.
    my $diskSpace = $resp->getValue("exec.out.diskSpace");

    $doc->dispose();
    $resp->dispose();

    if($diskSpace < ($disksize * 1024))    {
        my $diskSpaceGB = sprintf ("%.2f", $diskSpace / 1024);

        trace("Insufficient disk space $diskSpaceGB GB for " .
	      "disk of size $disksize GB", "warning");

        push(@warnings, "The disk size you have selected (" . $disksize .
                    " GB) is greater than the free space on the file system" . 
                    " (" . $disksize . " GB). The disk was created, but your" .
                    " virtual machine may run out of host disk space later.");
    }
}

sub CheckRAM
{
    my $memsize = shift ();

    my $total = &GetHostRAM;

    if($total < $memsize) {
        return "<exec><returncode>0</returncode></exec>";
    } else {
        return "<exec><returncode>1</returncode></exec>";
    }
}

sub GetHostRAM
{
    my $total;

    $total = $hostInfo->getValue("exec.out.memory.total");
    return $total;

}

sub CreateDisk
{
    my $stub = $p->profile();
    my $ret;
    my $d = shift();
    my $dskSize = shift();
    my $os = shift();
    my $scsi = shift();
    my $s;

    my $diskType = $scsi ? "scsi" : "ide";

    my $diskFileName = $d . $gDiskExtension; 
    my $configFileName = $d . $configFileExt;
    my $capacityGB = $dskSize;

    my $doc = VMware::DOMAccess->new("exec");
    
    $doc->setValue("exec.op", "CreateCOWDisk");
    $doc->setValue("exec.in.config", $configFileName);
    $doc->setValue("exec.in.filename", $diskFileName);
    $doc->setValue("exec.in.capacityGB", $capacityGB);
    $doc->setValue("exec.in.diskType", $diskType);
    $doc->setValue("exec.in.virtualHWversion", $gVirtualHWVersion);
    return request($doc);
}

sub WriteConfig($$)
{
    my $stub = $p->profile();
    my ($file, $data) = @_;
    my $doc = VMware::DOMAccess->newXML("<exec><op></op><in></in></exec>");
    $doc->setValue("exec.op", "File_Create");
    $doc->setValue("exec.in.file", $file);
    $doc->setValue("exec.in.data", $data);
    $doc->setValue("exec.in.mode", "0754");
    my $ret = request($doc);
    
    if(!OK($ret)) {
	if( PermissionDenied($ret) ) {
	    # XXX TODO: simplify...
	    return("<exec><returncode>0</returncode><messages><msg severity=\"error\">Permission denied</msg></messages></exec>");
	}
    }
    return($ret);
}

# Tests if the file or directory exists
sub FileExists
{
    my $file = shift;
    my $dir = shift;
    my $doc = VMware::DOMAccess->newXML("<exec><op></op><in></in></exec>");
    my $resp;
    my $resp_doc;
    my $result = 0;

    $doc->setValue("exec.op", "File_Exists");
    $doc->setValue("exec.in.file", $file);
    if (defined($dir)) { 
      $doc->setValue("exec.in.baseDirectory", $dir);
    }
    $resp = request($doc);
    if(!OK($resp)) {
	return(undef);
    }

    $resp_doc = VMware::DOMAccess->newXML($resp);

    # Make sure that the user has permissions to create a file in the directory
    $result = $resp_doc->getValue("exec.out.exists");
    $resp_doc->dispose();
    return($result);
}

sub deviceFileExists {
  my $dir = shift();
  my $file = shift();
  my $isDevice = shift();

  if ($isDevice && $^O eq "MSWin32") {
    if ($file =~ /^[a-zA-Z]:$/ || $file =~ /^auto detect$/i) {
      return 1;
    }
  } else {
    return FileExists($file, isAbsolutePath($file) ? undef : $dir);
  }
  return 0;
}

# Test if the user has write/access permissions on the directory
sub DirectoryIsEmpty
{
    my $dir = shift;
    my $doc = VMware::DOMAccess->newXML("<exec><op></op><in></in></exec>");
    my $resp;
    my $resp_doc;

    $doc->setValue("exec.op", "Directory_List");
    $doc->setValue("exec.in.directory", $dir);
    $resp = request($doc);
    if(!OK($resp)) {
	return(undef);
    }

    $resp_doc = VMware::DOMAccess->newXML($resp);

    # Make sure that the directory has no files other than . and ..
    my @files = $resp_doc->listElementNames("exec.out");
    my $file;
    foreach $file (@files) {
	my $name = $resp_doc->getValue("exec.out." . $file . ".name");
	if( $name ne '.' && $name ne '..' ) {
	    $resp_doc->dispose();
	    return(0);
	}
    }

    $resp_doc->dispose();
    return(1);
}

sub MakeConfigDirectory
{
    my $stub = $p->profile();
    my $dir = shift( );

    my $cdir = '';
    my $d;
    my $dirExists = 0;

    $dirExists = FileExists($dir);
    if( !defined($dirExists) ) {
	my $err = "Could not make directory $dir.  Failed while checking for the existence of the directory.";
	my $ret = "<exec><returncode>0</returncode><messages><msg severity=\"error\">$err</msg></messages></exec>";
	return($ret);
    }

    # Check to make sure that the directory does not exist or else it is empty.
    if($dirExists)
    {
	# Check to see if the directory is empty
	my $dirEmpty = DirectoryIsEmpty($dir);
	if( !defined($dirEmpty) ) {
	    my $err = "Could not make directory $dir.  Failed while checking if the directory is empty.";
	    my $ret = "<exec><returncode>0</returncode><messages><msg severity=\"error\">$err</msg></messages></exec>";
	    return($ret);
	}
	if( $dirEmpty ) {
	    # No need to create the directory
	    return "<exec><returncode>1</returncode></exec>";
	}

	my $err = qq(
The directory $dir already exists.  Specify another directory or remove the existing directory and try again.

); 
	my $ret = "<exec><returncode>0</returncode><messages><msg severity=\"error\">$err</msg></messages></exec>";
	
	return $ret;
    }

    my $doc = VMware::DOMAccess->newXML("<exec><op></op><in></in></exec>");
    $doc->setValue("exec.op", "Directory_Make");
    $doc->setValue("exec.in.directory", $dir);
    my $ret = request($doc);
    if(!OK($ret)) {
	if( PermissionDenied($ret) ) {
	    return("<exec><returncode>0</returncode><messages><msg severity=\"error\">Permission denied</msg></messages></exec>");
	}
    }
    return($ret);
}
 
sub GenerateConfig
{
    my $stub = $p->profile();
    my $query = new CGI;

    my $os = $query->param('os');
    my $dir = $query->param('dir');
    my $displayname = $query->param('displayname');
    my $memsize = $query->param('memsize');
    my $displayDepth = $query->param('displaydepth');
    my $networking = $query->param('networking');
    my $scsi = $query->param('scsi');
    my $cfg = "";
    my $floppyFileName = $query->param('floppy0FileName');
    my $floppyFileType = $query->param('floppy0FileType');
    my $floppyStartConnected = $query->param('floppy0StartConnected');
    my $cdromFileName = $query->param('cdromFileName');
    my $cdromDeviceType = $query->param('cdromDeviceType');
    my $cdromStartConnected = $query->param('cdromStartConnected');
    my $file;

    # XXX TODO
    # XXX This is wrong. We assume the MUI resides on the same machine as serverd!
    # Not to mention it doesn't work on windows.
    #path to vmware
    open(CONFIG, "/etc/vmware/config");
    my $key;
    my $vmbinpath;
    while(<CONFIG>)
    {

	($key, $vmbinpath) = split /=/;
	$key =~ s/ *//g;
	print STDERR ">> $_ ";
	print STDERR "  key: -$key- \n";
	if($key eq "vmware.fullpath") {
	    $vmbinpath = (split(/\"/, $vmbinpath))[1];
	    print STDERR "    path: -$vmbinpath- \n";
	    last;
	}
    }

    $cfg .= "#!$vmbinpath \n";

    #version
    my ($cvVar, $cvVal, $hvVar, $hvVal) = GetConfigVersion();
    $cfg .= "$cvVar = \"$cvVal\"\n";
    $cfg .= "$hvVar = \"$hvVal\"\n";
    $cfg .= "\n";

    $cfg .= "#OS\n";
    $cfg .= "displayName = \"$displayname\"\n"; 
    my $gos = ($os eq "winxp") ? "whistler" : $os;
    $cfg .= "guestOS = $gos\n";
    $cfg .= "\n";

    $cfg .= "#Virtual Disk \n";

    if (!$scsi) {
	$cfg .= "ide0.present = TRUE\n";
	$cfg .= "ide0:0.present = TRUE\n";
        $cfg .= "ide0:0.mode = persistent\n";
        $cfg .= "ide0:0.fileName = \"" . $os . $gDiskExtension . "\"\n";
        $cfg .= "\n";
    } else {
	$cfg .= "scsi0.present = TRUE\n";
	$cfg .= "scsi0:0.present = TRUE\n";
	$cfg .= "scsi0:0.deviceType = scsi-hardDisk\n";
	$cfg .= "scsi0:0.mode = persistent\n";
	$cfg .= "scsi0:0.fileName = \"" . $os . $gDiskExtension . "\"\n";
	$cfg .= "\n";
    }
 
    $cfg .= "#Memory size\n";
    $cfg .= "memsize = $memsize\n";
    $cfg .= "\n";

    $cfg .= "#Nvram\n";
    $cfg .= "nvram = \"" . $os . ".nvram\"\n";
    $cfg .= "\n";

    $cfg .= "#CD-ROM \n";
    $cfg .= "ide1:0.present = TRUE\n";
    $cfg .= "ide1:0.fileName = \"$cdromFileName\"\n";
    $cfg .= "ide1:0.deviceType = " .
      ($cdromDeviceType ? "cdrom-image" : "atapi-cdrom") . "\n";
    $cfg .= "ide1:0.startConnected = " .
      ($cdromStartConnected ? "TRUE" : "FALSE") . "\n";
    $cfg .= "\n";

    $file = $cdromFileName;
    if (! deviceFileExists($dir, $file, ! $cdromDeviceType)) {
      trace("No such CD-ROM: $file.", "warning");
      push(@warnings, "The CD-ROM drive or ISO image \"$file\" is not " .
           "accessible at this time. Please ensure that the file is " .
           "available before attempting to use it.");
    }

    $cfg .= "#Floppy\n";
    $cfg .= "floppy0.present = TRUE\n";
    $cfg .= "floppy0.fileName = $floppyFileName\n";
    $cfg .= "floppy0.fileType = " .
      ($floppyFileType ? "file" : "device") . "\n";
    $cfg .= "floppy0.startConnected = " .
      ($floppyStartConnected ? "TRUE" : "FALSE") . "\n";
    $cfg .= "\n";

    $file = $floppyFileName;
    if (! deviceFileExists($dir, $file, ! $floppyFileType)) {
      trace("No such floppy drive: $file.", "warning");
      push(@warnings, "The floppy drive or disk image \"$file\" is not " .
           "accessible at this time. Please ensure that the file is " .
           "available before attempting to use it.");
    }


    if($networking) {
	my $eth = "ethernet0";

	$cfg .= "#Networking\n";
        $cfg .= "$eth.present = TRUE\n";
	$cfg .= "$eth.connectionType = bridged\n";
    }
    $cfg .= "\n";

    $cfg .= "#Misc\n";
    $cfg .= "RemoteDisplay.depth = $displayDepth\n";
    $cfg .= "log.fileName = \"" . $os . ".log\"\n";
    if ($os eq "win2000") {
       $cfg .= "PciSvgaDynamic = TRUE\n";
    }

    return $cfg;
}

sub request($) {
    my $stub = $p->profile();
    my $doc = shift;
    my $err;

    my $xml = $doc->getValue("");
    my $ret = "";

    if ( $server && $server->is_connected() ) {
      $ret = $server->exec($xml);
      if (!$ret) {
        my ($err, $errstr) = $server->get_last_error();
        trace("VMControl error $err: $errstr\n", "error");
      }
    }

    return $ret;
}

sub ReturnError($)
{
    my $err = shift();
    my $query = new CGI;
    my $os = $query->param('os');
    my $displayname = $query->param('displayname');
    my $dir = $query->param('dir');
    my $memsize = $query->param('memsize');
    my $disksize = $query->param('disksize');
    my $networking = $query->param('networking');
    my $displaydepth = $query->param('displaydepth');
    my $scsi = $query->param('scsi');
    my $floppyFileName = $query->param('floppy0FileName');
    my $floppyFileType = $query->param('floppy0FileType');
    my $floppyStartConnected = $query->param('floppy0StartConnected');
    my $cdromFileName = $query->param('cdromFileName');
    my $cdromDeviceType = $query->param('cdromDeviceType');
    my $cdromStartConnected = $query->param('cdromStartConnected');

    my $doc = VMware::DOMAccess->new("input");

    $doc->setValue("input.error", $err);
    $doc->setValue("input.os", $os);
    $doc->setValue("input.displayName", $displayname);
    $doc->setValue("input.dir", $dir);
    $doc->setValue("input.diskSize", $disksize);
    $doc->setValue("input.memSize", $memsize);
    $doc->setValue("input.networking", $networking);
    $doc->setValue("input.displayDepth", $displaydepth);
    $doc->setValue("input.scsi", $scsi);
    $doc->setValue("input.floppy0.fileName", $floppyFileName);
    $doc->setValue("input.floppy0.fileType", $floppyFileType);
    $doc->setValue("input.floppy0.startConnected", $floppyStartConnected);
    $doc->setValue("input.cdrom.fileName", $cdromFileName);
    $doc->setValue("input.cdrom.deviceType", $cdromDeviceType);
    $doc->setValue("input.cdrom.startConnected", $cdromStartConnected);
    $doc->setValue("input.defaults.common.cdrom.filename", $gDefaultCDromFilename);
    $doc->setValue("input.defaults.common.floppy.filename", $gDefaultFloppyFilename);

    $doc = &AppendServerInfo($doc);
    $doc = &AppendDefaults($doc);
    my $xml = $doc->prettyPrint();

    return &emit_xml($xml);
}

sub ResponsePage
{
    my $stub = $p->profile();
    my $cfg = shift();
    my $xml;
    my $query = new CGI;
    my $warningCount = 0;

    my $os = $query->param('os');
    my $ram = $query->param('memsize');
    my $disk = $query->param('disksize');

    my $doc = VMware::DOMAccess->new("input");
    $doc->setValue("input.success.os", $os);
    $doc->setValue("input.success.ram", $ram);
    $doc->setValue("input.success.diskSize", $disk);
    $doc->setValue("input.success.cfg", $cfg);
    $doc = AppendServerInfo($doc);

    # It's important to drain the warnings since @warnings is a global and
    # will persist accross requests.
    while (scalar(@warnings)) {
      $doc->setValue("input.success.warning[$warningCount]", shift(@warnings));
      $warningCount++;
    }

    $xml = $doc->prettyPrint;
    
    return &emit_xml($xml);
}

1;
