#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# Constant.pm
#
# A class that contains the constants needed by the host configuration
# module.
#

package VMware::HConfig::Constant;
use strict;
use Exporter   ();
use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
use VMware::ExtHelpers qw(Panic Warning @BINPATH);

BEGIN {
    # set the version for version checking
    $VERSION     = 1.00;
    @ISA         = qw(Exporter);
    @EXPORT_OK   = qw(%gHelpers
		      %gFiles
                      %gCertInfo
		      %gDirs
                      %gBlockDeviceDir
                      %gBlockDeviceNames
		      %gHelpersInfo
		      &init
		      &c_getFile
		      &c_getDir
		      &c_getHelper
		      $gDefaultMemSize
                      $gMaxConsoleMemSize
		      @gRequiredDrivers
		      @gOptionalDrivers
		      %gDriverSpecific
		      $gVmkDumpPartSize
		      $gVmkDumpPartType
		      $gVMFSPartType
		      $gVMFSMaxKBlocks
		      $gVMFS2MaxKBlocks
		      $gVMFSBlockSizeLimit
		      $gVMFS2BlockSizeLimit
                      $gVMFSAbsMaxFileSize
                      $gVMFS2AbsMaxFileSize
                      &CACHE_NEVER
		      &CACHE_ONCE
		      &CACHE_CALLBACK
		      $gSSHost_CacheDefaults
		      $gSSHost_CacheSource
		      $gSSHost_CacheRemote
		      $gRawDisk_CacheDefaults
		      $gSSHostFlushPeriod
		      @gPartitionCreateTypes
		      $gVmkDeviceType_nic
		      $gVmkDeviceType_bond
		      $gVmkDeviceType_tcpip
		      $gVmkDeviceType_migration
		      $gVmkDeviceType_hba
		      %gNonDiskSCSITypes
                      @gMultipathPolicies

		      @gVmnixHostAdapterNames

                      @gNICSettings
                      %gNICConfigurationOptions
                      @gNICClusteringDisabled

		      $URL_INDEX
		      $URL_PCIDIVY
		      $URL_VMCONFIG
		      $URL_VIEWPART
                      $URL_NICCONFIG
                      $URL_LICENSE
                      $URL_SWAPFILE
		      $URL_WIZ
		      $URL_LOGOUT
		      $URL_OVERVIEW
		      $URL_HSTATUS
                      $URL_SECURITY
                      $URL_REBOOT

		      $gUseSIGCHLD
                      $CFG_HOST_SECURITY
                      $CFG_CERTIFICATE

                      %EsxScriptedInstall
            );

    if( -d '@@PREFIX@@' ) {
	push(@BINPATH, '@@PREFIX@@/apache/bin');
    } else {
	# Default location especially useful for devel builds
	push(@BINPATH, '/usr/lib/vmware-mui/apache/bin');
    }

}

use vars @EXPORT_OK;

%gFiles = ( 'proc_pci' => '/proc/pci',                   # PCI info on Linux/vmnix
	    'proc_cmdline' => '/proc/cmdline',           # Command line args passed in from LILO
	    'lilo.conf' => '/etc/lilo.conf',             # LILO configuration info
	    'snmpd.conf' => '/etc/vmware/vmware-snmpd.conf', # SNMP configuration info
	    'mtab' => '/etc/mtab',                       # Mounted directories
	    'pcidev_map' => '/etc/vmware/vmware-devices.map', # Maps devices to vmk drivers
	    'pcidev_map_local' => '/etc/vmware/vmware-devices.map.local', # Custom map of devices to vmk drivers
            'modules_conf' => '/etc/modules.conf',       # Linux/vmnix module config
	    'proc_cpuinfo' => '/proc/cpuinfo',		 # CPU info
	    'proc_meminfo' => '/proc/meminfo',           # Memory info
	    'proc_vmware_meminfo' => '/proc/vmware/mem', # Memory info the vmkernel sees
            'proc_vmware_vmkstor' => '/proc/vmware/vmkstor', # Table of all devices
	    'config_db' => '/etc/vmware/hwconfig',       # Cache file detailing hardware
	    'host_config' => '/etc/vmware/config',       # Host-wide preference file
            'vmkernel_config' => '/etc/vmware/vmkconfig', # VMkernel config settings
	    'locations' => '/etc/vmware/locations',      # Locations database
	    'vmkmodule' => '/etc/vmware/vmkmodule.conf', # List of vmk modules to load
	    'eula' => '/usr/share/doc/vmware/EULA',      # Location of the EULA
	    'dhcp_copyright' => '/usr/share/doc/vmware/DHCP-COPYRIGHT',  # Location of DHCP Copyright
	    'samba_license' => '/usr/share/doc/vmware/SAMBA-LICENSE',  # Location of Samba License
            'inetd_config' => '/etc/inetd.conf',         # inetd services config file
            'swap_stats' => '/proc/vmware/swap/stats',   # VMkernel swap stats.
            'mem_sched' => '/proc/vmware/sched/mem',     # VMkernel mem sched.
            'devnames_conf' => '/etc/vmware/devnames.conf',  # VMKernel device names config.
            'proc_partitions' => '/proc/partitions'      # List of scsi devices
            );

%gDirs = ( 'boot' => '/boot',                          # Boot files are found can be found
	   'dev' => '/dev',                            # Devices directory
	   'proc_scsi' => '/proc/scsi',                # SCSI information for the vmnix
	   'proc_bus_pci' => '/proc/bus/pci',          # Raw PCI configuration space info
	   'proc_vmware' => '/proc/vmware',            # Proc entry created when vmkernel is loaded
	   'proc_vmware_scsi' => '/proc/vmware/scsi',  # SCSI information for the vmkernel
	   'proc_vmware_net' => '/proc/vmware/net',    # Network interfaces for the vmkernel
	   'vmkmodule' => '/usr/lib/vmware/vmkmod',    # Directory where vmk modules can be found
	   'cpu' => '/proc/vmware/sched',              # Directory where CPU info can be found
	   'proc_vm' => '/proc/vmware/vm',             # Directory where world info can be found
	   'proc_filters' => '/proc/vmware/filters',   # Directory where filter info can be found
           'init_scripts' => '/etc/rc.d/init.d',       # Directory that init scripts live in
	   'proc_vmk_config' => '/proc/vmware/config', # vmkernel global config options
           'vmfs' => '/vmfs',                          # Contains list of VMFS

	   # Device Specific infomation
	   'dev_ida' => '/dev/ida',                    # Directory where ida blk device can be found
	   'dev_cciss' => '/dev/cciss',                # Directory where cciss blk device can be found
	   'dev_rd' => '/dev/rd',                      # Directory where rd blk device can be found
	   );

%gBlockDeviceDir = ( 'cpqarray.o' => c_getDir('dev_ida'),
                     'cciss.o' => c_getDir('dev_cciss'),
                     'DAC960.o' => c_getDir('dev_rd'),
                     );

%gBlockDeviceNames = ('rd' => 'rd',
                      'cciss' => 'cciss',
                      'ida' => 'ida'
                      );

%gHelpersInfo = ( 'echo' => ['echo'],
		  'copy' => ['cp'],
		  'sed' => ['sed'],
		  'safe_lspci' => ['lspci', '-vn'],
		  'safe_lspci_names' => ['lspci', '-v'],
		  'probe_lspci' => ['lspci', '-H1', '-M', '-vn'],
		  'probe_lspci_names' => ['lspci', '-H1', '-M', '-v'],
		  'probe_lspci2' => ['lspci', '-H2', '-M', '-vn'],
		  'probe_lspci2_names' => ['lspci', '-H2', '-M', '-v'],
		  'lilo' => ['lilo'],
		  'uname' => ['uname', '-a'],
		  'lsmod' => ['lsmod'],
		  'list_vmkmod' => ['vmkload_mod', '-l'],
		  'load_vmkmod' => ['vmkload_mod'],
		  'partition_create' => ['vmware-sfdisk', '-D', '-L', '--force'],
		  'partition_create_beta2compat' => ['vmware-sfdisk', '-L', '--force'],
		  'partition_settype' => ['vmware-sfdisk', '--change-id'],
		  'partition_list' => ['vmware-sfdisk', '-l'],
		  'partition_list_sect' => ['vmware-sfdisk', '-d'],
		  'disk_geometry' => ['vmware-sfdisk', '-g'],
                  'get_san_info' => ['wwpn.pl', '-d'],
		  'vmfs_create_fs' => ['vmkfstools', '--force', '-C'],
		  'vmfs_bind_vsd' => ['vmkfstools', '--force', '-N'],
		  'vmfs_list_files' => ['vmkfstools', '--force', '-l'],
		  'vmfs_list_partitions' => ['vmkfstools', '--force', '-P'],
		  'vmfs_export_file' => ['vmkfstools', '--force', '--machine-readable', '-e'],  #XXX should use disklib!
		  'vmfs_import_file' => ['vmkfstools', '--force', '--machine-readable', '-i'],  #XXX should use disklib!
		  'vmfs_remove_file' => ['rm', '-f'],
		  'vmfs_create_file' => ['vmkfstools', '--force', '-c'],
		  'vmfs_change_owner' => ['chown', '-f'],
		  'vmfs_change_access' => ['chmod', '-f'],
		  'vmfs_set_name' => ['vmkfstools', '--force', '-S'],
                  'vmfs_set_accessibility' => ['vmkfstools', '--force', '-F'],
                  'vmfs_attach_extent' => ['vmkfstools', '--force', '-Z'],
                  'vmfs_rescan' => ['vmkfstools', '--force', '-V'],
                  'vmfs_convert_vmfs2' => ['vmkfstools', '--force', '-T'],
                  'lun_rescan' => ['vmkfstools', '--force', '-s'],
                  'lun_IO' => ['fdisk'],
                  'persist_adapter' => ['pbind.pl', '-p'],
                  'wwpn_list' => ['wwpn.pl'],
                  'start_swapper' => ['vmkfstools', '--force', '-w'],
		  'vmware_config' => ['vmware-config.pl'],
		  'vmkdump_set' => ['vmkdump', '--partition'],
		  'vmkdump_unset' => ['vmkdump', '--partition', 'none:0:0:0'],
		  'reboot' => ['/sbin/reboot'],
                  'halt' => ['/sbin/halt'],
		  'license_check' => ['vmware-vmx', '--can-run'],
		  'license_register' => ['vmware-vmx', '--new-sn'],
		  'license_info' => ['vmware-vmx', '--query-license'],
                  'host_service_set' => ['chkconfig','--level','3'],
                  'host_service_list' => ['chkconfig','--list'],
                  'net_stat' => ['netstat', '-lt', '--program'],
                  'killall' => ['killall'],
                  'restart_inetd' =>  ['killall','-HUP','inetd'],
                  'restart_xinetd' =>  ['killall','-USR2','xinetd'],
                  'logger' => ['logger', '-p', 'local6.info'],
                  'mkinitrd' => ['mkinitrd', '-f'],
		  );
%gHelpers = ();



#holding defaults for the certificate info
# The max lengths are from openssl (others fields may also be so limited).
%gCertInfo =
   ( countryname => {value=>"US", msg=> "Two letter country code", maxLen=>2},
     stateorprovincename => {value=>"", msg=>"State or Province" },
     localityname => {value=>"", msg=> "City"},
     organizationname => {value=>"", msg=>"Company/Organization Name"},
     organizationalunitname => {value=>"", msg=> "Department Name"},
     commonname => {value=>"", msg=>"Server's hostname" , maxLen=>64},
     emailaddress => {value=>"", msg=>"Administrator's email address", maxLen=>40},
     daysvalid => {value=>"365", msg=>"Number of days before certificate expires"},
   );

# Default and maximum memory sizes to use for VMnix
$gDefaultMemSize = 272;
$gMaxConsoleMemSize = 800;

# The default drivers that are installed
@gRequiredDrivers = ( { MODULE => "vmklinux",
			TYPE => "linux" } );

# The default drivers that are installed
@gOptionalDrivers = ( { MODULE => "nfshaper.o",
			TYPE => "nfshaper" } );


# List if directories that need to be checked when looking for the
# Vmnix SCSI adapters.  see SSHost->findVmnixScsiAdapters() for more
# information.
%gDriverSpecific = ( 'aic7xxx.o' => 'proc_aic7xxx',
		     'aic79xx.o' => 'proc_aic79xx',
		     'ncr53c8xx.o' => 'proc_ncr53c8xx',
		     'BusLogic.o' => 'proc_buslogic',
		     'aacraid.o' => 'proc_aac'
		     );

# The size of the VmkDump partition in Kbytes
$gVmkDumpPartSize = 100 * 1024;
$gVmkDumpPartType = 0xfc;

$gVMFSPartType = 0xfb;

# How many kilo blocks a VMFS file can have
$gVMFSMaxKBlocks = 144;
$gVMFS2MaxKBlocks = 456;

# Configured limit on the maximum block size (MB)
$gVMFSBlockSizeLimit = 16;
$gVMFS2BlockSizeLimit = 64;

# VMFS-1 files are limited to a maximum size of 2 T (2048 G).
# VMFS-2 file sizes are limited to approximately 64 T.
$gVMFSAbsMaxFileSize = 2048;
$gVMFS2AbsMaxFileSize = 65536;

# Types of devices in the vmkernel
$gVmkDeviceType_nic = 'vmnic';
$gVmkDeviceType_bond = 'bond';
$gVmkDeviceType_tcpip = 'tcpip';
$gVmkDeviceType_migration = 'migration';
$gVmkDeviceType_hba = 'vmhba';

# These are bindings for Non-Disk SCSI device types
%gNonDiskSCSITypes = ('Direct-Access' => 'ignore',
		      'Processor' => 'ignore',
		      'CD-ROM' => 'CDROM',
		      'Sequential-Access' => 'Tape'
		      );

# List of valid multipathing policies
@gMultipathPolicies = ('fixed', 'rr', 'mru');

# Flag to indicate if SIGCHLD is being used (ie. not ignored)
$gUseSIGCHLD = 1;

# Used to indicate if init has been called before
my $gInitFlag = 0;

# Caching constants
sub CACHE_NEVER { 0 };       # Always regenerate information
sub CACHE_ONCE { 1 };        # Never regenerate information
sub CACHE_CALLBACK { 2 };    # Flush the cache information when the cache callback is called

# These are what we probably want for Beta 3
$gSSHost_CacheDefaults = { 'KERNEL_INFO' =>       CACHE_ONCE,
			   'PCIDEVICE_MAP' =>     CACHE_ONCE,
			   'PCIDEVICES' =>        CACHE_CALLBACK,
                           'DEVICENAMES' =>       CACHE_CALLBACK,
			   'LILOCONF' =>          CACHE_ONCE,
			   'SNMPCONF' =>          CACHE_NEVER,
			   'BIMAGE_FILES' =>      CACHE_ONCE,
			   'INITRD_FILES' =>      CACHE_ONCE,
			   'DIR_MAP' =>           CACHE_ONCE,
			   'VMK_SCSI_ADAPTERS' => CACHE_CALLBACK,
			   'VMK_NET_ADAPTERS' =>  CACHE_NEVER,
			   'VMK_NET_BOND_ADAPTERS' =>  CACHE_NEVER,
			   'MFS' =>               CACHE_NEVER,
			   'NUM_CPUS' =>          CACHE_ONCE,
			   'TOTAL_MEMSIZE' =>     CACHE_NEVER,
			   'VMNIX_MEMSIZE' =>     CACHE_NEVER,
			   'VMKDUMP_PARTITION' => CACHE_NEVER,
			   'CURRENT_LABEL' =>     CACHE_ONCE,
			   'CONFIG_DB' =>         CACHE_ONCE,
			   'LOCATIONS_DB' =>      CACHE_ONCE,
			   'RUNNING_WORLDS' =>    CACHE_NEVER
			   };

# Format of each entry
# ->{cache_name}->[0] := name of operation
# ->{cache_name}->[1] := time of last access
$gSSHost_CacheSource = { 'PCIDEVICE_MAP' => [c_getFile('pcidev_map')],
			 'PCIDEVICES' => [c_getFile('config_db')],
                         'DEVICENAMES' => [c_getFile('config_db')],
			 'LILOCONF' => [c_getFile('lilo.conf')],
			 'BIMAGE_FILES' => [c_getDir('boot')],
			 'INITRD_FILES' => [c_getDir('boot')],
			 'DIR_MAP' => [c_getFile('mtab')],
			 'CONFIG_DB' => [c_getFile('config_db')],
			 'LOCATIONS_DB' => [c_getFile('locations')]
			 };

# These are what we need for Beta 2.  Performance will be slower
#$gSSHost_CacheDefaults = { 'KERNEL_INFO' =>       CACHE_ONCE,
#			   'PCIDEVICE_MAP' =>     CACHE_ONCE,
#			   'PCIDEVICES' =>        CACHE_NEVER,
#			   'LILOCONF' =>          CACHE_CALLBACK,
#			   'BIMAGE_FILES' =>      CACHE_CALLBACK,
#			   'INITRD_FILES' =>      CACHE_CALLBACK,
#			   'DIR_MAP' =>           CACHE_NEVER,
#			   'VMK_SCSI_ADAPTERS' => CACHE_NEVER,
#			   'VMK_NET_ADAPTERS' =>  CACHE_NEVER,
#			   'MFS' =>               CACHE_NEVER,
#			   'NUM_CPUS' =>          CACHE_ONCE,
#			   'TOTAL_MEMSIZE' =>     CACHE_NEVER,
#			   'VMNIX_MEMSIZE' =>     CACHE_NEVER,
#			   'VMKDUMP_PARTITION' => CACHE_NEVER,
#			   'CURRENT_LABEL' =>     CACHE_ONCE,
#			   'CONFIG_DB' =>         CACHE_CALLBACK
#			   'RUNNING_WORLDS' =>    CACHE_NEVER,
#			   };

# These are what we use for the remote version of SSHost.
$gSSHost_CacheRemote = { 'KERNEL_INFO' =>       CACHE_ONCE,
			 'PCIDEVICE_MAP' =>     CACHE_ONCE,
			 'PCIDEVICES' =>        CACHE_ONCE,
                         'DEVICENAMES' =>       CACHE_ONCE,
			 'LILOCONF' =>          CACHE_ONCE,
			 'BIMAGE_FILES' =>      CACHE_ONCE,
			 'INITRD_FILES' =>      CACHE_ONCE,
			 'DIR_MAP' =>           CACHE_ONCE,
			 'VMK_SCSI_ADAPTERS' => CACHE_ONCE,
			 'VMK_NET_ADAPTERS' =>  CACHE_ONCE,
			 'VMK_NET_BOND_ADAPTERS' =>  CACHE_ONCE,
			 'MFS' =>               CACHE_ONCE,
			 'NUM_CPUS' =>          CACHE_ONCE,
			 'TOTAL_MEMSIZE' =>     CACHE_ONCE,
			 'VMNIX_MEMSIZE' =>     CACHE_ONCE,
			 'VMKDUMP_PARTITION' => CACHE_ONCE,
			 'CURRENT_LABEL' =>     CACHE_ONCE,
			 'CONFIG_DB' =>         CACHE_ONCE,
			 'LOCATIONS_DB' =>      CACHE_ONCE,
			 'RUNNING_WORLDS' =>    CACHE_ONCE
			 };

$gRawDisk_CacheDefaults = { 'GEOMETRY' =>            CACHE_ONCE,
			    'PARTITION_TABLE' =>     CACHE_ONCE,
                            'MULTIPATH_INFO' =>      CACHE_NEVER,
			    };

# Lists of valid settings for vmnic devices and the corresponding
# options to be passed to each module.
#
# undef => setting is not available for specified device.
# empty string => setting is the default -- no options are required.

@gNICSettings = ( "Autonegotiate",
                  "10 Mbps, half duplex",
                  "10 Mbps, full duplex",
                  "100 Mbps, half duplex",
                  "100 Mbps, full duplex",
                  "1000 Mbps, half duplex",
                  "1000 Mbps, full duplex",
                  );

$gNICConfigurationOptions{'e100.o'} = [ "e100_speed_duplex=0",
                                        "e100_speed_duplex=1",
                                        "e100_speed_duplex=2",
                                        "e100_speed_duplex=3",
                                        "e100_speed_duplex=4",
                                        ];

$gNICConfigurationOptions{'e1000.o'} = [ "Speed=0 Duplex=0",
                                         "Speed=10 Duplex=1",
                                         "Speed=10 Duplex=2",
                                         "Speed=100 Duplex=1",
                                         "Speed=100 Duplex=2",
                                         "Speed=1000 Duplex=1",
                                         "Speed=1000 Duplex=2",
                                         ];

$gNICConfigurationOptions{'acenic.o'} = [ "",
                                          "link=0x213",
                                          "link=0x212",
                                          "link=0x223",
                                          "link=0x222",
                                          "link=0x243",
                                          "link=0x242",
                                          ];

$gNICConfigurationOptions{'bcm5700.o'} = [ "line_speed=0",
                                           "line_speed=10 auto_speed=0 full_duplex=0",
                                           "line_speed=10 auto_speed=0 full_duplex=1",
                                           "line_speed=100 auto_speed=0 full_duplex=0",
                                           "line_speed=100 auto_speed=0 full_duplex=1",
                                           "line_speed=1000 auto_speed=0 full_duplex=0",
                                           "line_speed=1000 auto_speed=0 full_duplex=1",
                                           ];

$gNICConfigurationOptions{'3c990.o'} = [ "force=4",
                                         "force=0",
                                         "force=1",
                                         "force=2",
                                         "force=3",
                                         ];

#Driver only allows forcing half / full if you select a media type.
#I don't know the difference between 100_BASE_TX and 100_BASE_FX,
#so I'm assuming our customers don't either. -- greG
$gNICConfigurationOptions{'3c90x.o'} = [ "",
                                         undef,
                                         undef,
                                         undef,
                                         undef,
                                         undef,
                                         undef,
                                         ];

$gNICConfigurationOptions{'3c59x.o'} = $gNICConfigurationOptions{'3c90x.o'};

$gNICConfigurationOptions{'tulip.o'} = $gNICConfigurationOptions{'3c90x.o'};

# Don't allow clustering to be enabled for GigE -- it doesn't help
@gNICClusteringDisabled = ( 'e1000.o', 'acenic.o', 'bcm5700.o' );

# Flush the cached data in SSHost every 15 minutes.  This ensures that
# we periodically go back to the hardware for information about our
# data.  The 15 minutes is merely a bit of insurance against situations
# where we fail to detect a change in the configuration of the machine.
#
# XXX Currently, the caching won't help us much as most requests from the
#     SSHostRemote class just grab everything.  In the future, SSHostRemote
#     should just request data on a demand basis.
$gSSHostFlushPeriod = 15 * 60;

# The partition types that users are allowed to create
@gPartitionCreateTypes = ($gVMFSPartType, $gVmkDumpPartType, 0x05);

# These are the URLs used in the configuration process
$URL_INDEX = "/vmware/config";
$URL_PCIDIVY = "/pcidivy";
$URL_VMCONFIG = "/vmware-config";
$URL_VIEWPART = "/viewpart";
$URL_NICCONFIG = "/network-config";
$URL_LICENSE = "/license";
$URL_SWAPFILE = "/swapfile";
$URL_WIZ = "/vmcfg-esx";
$URL_LOGOUT = "/logout";
$URL_OVERVIEW = "/overview";
$URL_HSTATUS = "/hstatus";
$URL_SECURITY = "/security-config";
$URL_REBOOT = "/reboot";

@gVmnixHostAdapterNames = ("scsi", "ida", "cciss", 'c');


#security configuration config namespace
$CFG_HOST_SECURITY = "security.host.";
$CFG_CERTIFICATE = "security.mui_certificate.";

## ESX Scripted install constants
%EsxScriptedInstall = ( ## File locations
                        'HTDOC_ROOT' => '/usr/lib/vmware-mui/apache/htdocs',
                        'WEBCONFIG_ROOT' => '/usr/lib/vmware-mui/lib/perl5/site_perl/5.005/VMware/Management/SS/ScriptedInstall',

                        'SUDOERS' => '/etc/sudoers',
                        'BOOTCD_LOCATION' => '/var/vmware',
                        'ESXCD_LOCATION' => '/mnt/cdrom/VMware/RPMS',
                        'BOOTIMG_LOCATION' => '/var/vmware/images/boot.img',
                        'BOOTIMG_NET_LOCATION' => '/var/vmware/images/bootnet.img',
                        'RESERVE_SPACE' => 50,
                        'CDROM_ISO_SIZE' => 470,

                        'POST_INSTALL' => 'post-install.cfg',
                        'KS_FILE' => 'ks.cfg',
                        'KS_VERSION' => 720,
                        'DEFAULT_DISKIMAGE_NAME' => 'bootdisk.img',
                        'SUDO_ENTRY' => "
nobody ALL=NOPASSWD:/usr/lib/vmware-mui/lib/perl5/site_perl/5.005/VMware/Management/SS/ScriptedInstall/modifyboot
",

                        #errors
                        'ERRORS' => { 'ENOTSETUP' => -1,
                                      'EALREADYSETUP' => -2,
                                      'EFREESPACE' => -3,
                                      'ENOTMOUNTED' => -4,
                                      'ETMPNAME' => -5,
                                      'ENOSUDO' => -6,
                                    },

                        ## Network info 
                        'PORT' => 8555,
                        'MAX_PARTITIONS' => 10,

                        'SETTINGS' => {
                          'keyboard' => {
                            'file' => '/etc/sysconfig/keyboard',
                            'translate' => {
                              'keytable' => 'keyboard'
                            }
                          },
                        
                          'tz' => {
                            'file' => '/etc/sysconfic/clock',
                            'translate' => {
                              'zone' => 'timezone'
                            }
                          },
                          'i18n' => {
                            'file' => '/etc/sysconfig/i18n',
                          },
                          'mouse' => {
                            'file' => '/etc/sysconfig/mouse',
                            'translate' => {
                              'mousetype' => 'mouse'
                            },
                          }
                        }
                      );

# Must be called after perl parse time.  In other words, this init()
# function cannot be called in a BEGIN subroutine.  Fortunately, the
# accessor functions all call init if it has not yet been called.
sub init {
    if( $gInitFlag ) {
	return;
    }
    $gInitFlag = 1;

    # Build the list of helpers
    %gHelpers = &VMware::ExtHelpers::build_helpers(%gHelpersInfo);

    # The following checks don't really do anything.  They just print
    # out debug information to the log

    # Verify the files by doing a crude check for existence and
    # readability.  When we actually use the file, we may have to
    # perform some domain specific checks.
    &VMware::ExtHelpers::verify_files(values(%gFiles));

    # Verify the directories.  Use a crude check for readability and
    # accessibility.
    &VMware::ExtHelpers::verify_dirs(values(%gDirs));
}

# Gets a file given a sybolic name
sub c_getFile {
    my $name = shift;
    if( !$gInitFlag ) {
	init();
    }
    if( defined($gFiles{$name}) ) {
	# Use the symlink information
	if( -l $gFiles{$name} ) {
	    my $link = readlink($gFiles{$name});
	    if( defined($link) && -f $link ) {
		# Untaint data
		if( $link =~ /^(\/.*)/ ) {
		    $link = $1;
		}
		return($link);
	    }
	}
	return($gFiles{$name});
    } else {
	Panic('Requesting an invalid file "' . $name . '".');
    }
}

# Gets a directory given a symbolic name
sub c_getDir {
    my $name = shift;
    if( !$gInitFlag ) {
	init();
    }
    if( defined($gDirs{$name}) ) {
	# Use the symlink information
	if( -l $gDirs{$name} ) {
	    my $link = readlink($gDirs{$name});
	    if( defined($link) && -d $link ) {
		# Untaint data
		if( $link =~ /^(\/.*)/ ) {
		    $link = $1;
		}
		return($link);
	    }
	}
	return($gDirs{$name});
    } else {
	Panic('Requesting an invalid directory "' . $name . '".');
    }
}

#  Given a symbolic name, gets the incantation needed for a help
sub c_getHelper {
    my $name = shift;
    if( !$gInitFlag ) {
	init();
    }
    if( defined($gHelpers{$name}) ) {
	return($gHelpers{$name});
    } else {
	Warning('Requesting an invalid helper "' . $name . '".');
	return('');
    }
}


1;
